(in-package bnet)


(defparameter *rel-vars-mutual-information-threshold* .1
  "If the normalized conditional mutual information information is less than this, the variables are considered independent in relevant-variables")
  

(defun relevant-variables (pairs vars &key y-vars)
  "relevant-variables PAIRS VARS &key (Y-VARS #'identity).   PAIRS is a vector of transition examples (X . Y).  VARS is an alist from variable names to functions of the X variable.  This function searches for a minimal-size subset S of the VARS such that S renders the remaining variables in VARS conditionally independent of Y.  Conditional independence is tested using the conditional mutual information with threshold *rel-vars-mutual-information-threshold*"
  
  (break  "This code no longer guaranteed to work due to changes in empirical-dist and set equality checking code.")
  (flet ((get-x-vars (names)
	   (mapcar #'(lambda (name) (compose (mapping:evaluate vars name) #'car)) names)))
    (let ((d (empirical-dist pairs :test #'equalp))
	  (var-names (mapcar #'car vars))
	  (y-vars (mapcar #'(lambda (v) (compose v #'cdr)) (designated-list y-vars))))
      (do-elements (s (make-instance '<powerset> :base-set var-names :iteration-order ':increasing-size) (assert nil))
	(let ((u (complement-set s :base-set var-names :result-type ':list)))
	  (when (< (normalized-conditional-mutual-information d y-vars (get-x-vars u) (get-x-vars s)) *rel-vars-mutual-information-threshold*)
	    (return s)))))))


(defun simple-2tbn-structure-learning (pairs vars)
  "simple-2tbn-structure-learning PAIRS VARS.  VARS is an alist from variable names to functions.  For each variable V of VARS, look for the minimal subset of variables S in the previous time step that V is conditionally independent of the remaining variables given S, using the function relevant-variables.  Return an association list mapping each variable to its parents."
  (mapping:transform vars #'(lambda (name fn) (declare (ignore name)) (relevant-variables pairs vars :y-vars fn))))