;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; test script validating difference updating on simple
;; MDP
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defpackage test-difference-updating
  (:use
   cl
   graph
   prob
   set
   utils
   mdp
   diff-fn
   mdp-env
   diff-up)
  (:nicknames test-diff-up)
  )

(in-package test-difference-updating)

(defun make-chain-mdp (n p)
  (let ((trans (make-array (list n 2 n) :initial-element 0))
	(rewards (make-array n :initial-element -1))
	(term (make-array n :initial-element nil))
	(q (- 1 p)))
    (setf (aref term 0) t)
    (for-loop (i 1 (1- n))
      (setf (aref trans i 0 (1- i)) q
	    (aref trans i 0 (1+ i)) p
	    (aref trans i 1 (1- i)) p
	    (aref trans i 1 (1+ i)) q))
    (setf (aref trans 0 0 0) 1
	  (aref trans 0 1 0) p
	  (aref trans 0 1 1) q
	  (aref trans (1- n) 1 (1- n)) 1
	  (aref trans (1- n) 0 (1- n)) p
	  (aref trans (1- n) 0 (- n 2)) q)
    
    (make-tabular-mdp trans rewards :termination-vector term :init-dist (make-deterministic-dist (1- n)))))


(defvar *n* 4)
(defvar *eta* .5)
(setf m (make-chain-mdp *n* .1)
      table (make-array (list *n* *n*) :element-type 'float :initial-element 0.0)
      adjacent-pairs (append (mapset 'list #'(lambda (i) (cons i (1+ i))) (1- *n*))
			     (mapset 'list #'(lambda (i) (cons (1+ i) i)) (1- *n*)))
      d (make-instance '<tabular-difference-function> :states *n* :table table)
      g (state-transition-graph m))

(defun bup ()
  (dolist (p adjacent-pairs)
    (diff-up:bellman-update d (car p) (cdr p) m *eta*)))

(defun nup ()
  (dolist (p adjacent-pairs)
    (diff-up:normalize d (make-array 2 :initial-contents (list (car p) (cdr p))) *eta*)))

(defparameter *valid-ranges* 
    '((0 0 0 0)
      (3 3 0 0)
      (0 2 0 0)
      (0 1 0 0)
      (1 0 .7 1.3)
      (2 1 .7 1.3)
      (3 2 .7 1.3)
      (2 3 -1.3 -.7)
      (1 2 -1.3 -.7)))

(defun test-valid (f target)
  (every #'(lambda (x)
	     (dbind (s0 s1 l h) x
	       (between (difference f s0 s1 0) l h)))
	 target))

(format t "~&Note that these are randomized tests, so will occasionally fail even if the code is correct.")

(tests 
 "Difference updating"
 ((difference-updating m :normalization-frequency .5 :step-size .4 :max-num-steps 1000)
  *valid-ranges* :test #'test-valid)
 ((difference-updating m :normalization-frequency .9 :step-size .4 :max-num-steps 5000)
  *valid-ranges* :test #'test-valid)
 ((difference-updating m :normalization-frequency .1 :step-size .4 :max-num-steps 1000)
  *valid-ranges* :test #'test-valid)
 ((difference-updating m :normalization-frequency .5 :step-size .4 :normalization-step-size .9 :max-num-steps 1000)
  *valid-ranges* :test #'test-valid))