(in-package depots)


; TODO: what to do about equivalence of trucks (roughly) and hoists per loc (exactly)

; TODO: this is not optimality-preserving since it ignores moves without a truck
; These would not be necessary with numeric costs (except if no truck needs to go to loc)
;   but in propositional version they are

(defun make-depots-hierarchy (d)
  (let ((blocks     (lookup-type d 'crates))
	(trucks     (lookup-type d 'trucks))
        (locations  (lookup-type d 'locations))
        (surfaces   (lookup-type d 'surfaces))
        (hoists     (lookup-type d 'hoists)))
    (let ((slurp (make-hla-schema
		  :var-domains `((?t . ,trucks) (?l . ,locations))
		  :top-level-precond '(and (at ?t ?l))
		  :implementations (list
				    (make-implementation 
				     :precond '(and)
				     :actions '())
				    (make-implementation
				     :var-domains `((?b . ,blocks) (?c . ,surfaces) (?h . ,hoists))
				     :precond '(and (clear ?b) (at ?b ?l) (on ?b ?c) (available ?h) (at ?h ?l))
				     :actions '((lift ?h ?b ?c ?l) (load ?h ?b ?t ?l) (slurp ?t ?l))))))

	  (blurp (make-hla-schema
		  :var-domains `((?t . ,trucks) (?l . ,locations))
		  :top-level-precond '(and (at ?t ?l))
		  :implementations (list
				    (make-implementation 
				     :precond '(and)
				     :actions '())
				    (make-implementation
				     :var-domains `((?b . ,blocks) (?c . ,surfaces) (?h . ,hoists))
				     :precond '(and (clear ?c) (at ?c ?l) (in ?b ?t) (available ?h) (at ?h ?l))
				     :actions '((unload ?h ?b ?t ?l) (drop ?h ?b ?c ?l) (blurp ?t ?l))))))

	  (step  (make-hla-schema 
		     :var-domains `((?t . ,trucks) (?l . ,locations) (?l2 . ,locations))
		     :top-level-precond '(and (at ?t ?l1))
		     :implementations (list 
				       (make-implementation 
					:precond '(and); (not (equal ?l ?l2)))
					:actions '((slurp ?t ?l) (blurp ?t ?l) (drive ?t ?l ?l2))))))

	  (finish  (make-hla-schema 
		     :var-domains `((?t . ,trucks) (?l . ,locations))
		     :top-level-precond '(and (at ?t ?l1))
		     :implementations (list 
				       (make-implementation 
					:precond '(and)
					:actions '((slurp ?t ?l) (blurp ?t ?l))))))) ;; TODO: destroy truck here
      
      (make-instance '<prop-hierarchy> 
	:planning-problem d
	:top-level-actions '(step)
	:refinement-order '((step finish) (slurp blurp))
	:hla-schemas (p2alist  'step step 'finish finish 'slurp slurp 'blurp blurp)))))










#|
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Implemented ops from <hierarchy>
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defmethod applicable-top-level-actions (h s)
  (let ((locations (locs h))
	(misplaced (dcs-loc-misplaced s)))
    (let ((loc-pairs (make-instance '<prod-set> :sets (list locations locations) :iterate-quickly t)))
      (disjoint-union
       
       ;; transfer actions
       (ndlet-fail ((x loc-pairs))
	 (dsbind (src dst) x
	   (aif (crates-needed src dst s h)
	       (list 'transfer-crates src dst it)
	     'fail)))
   
       ;; rearrange actions
       (ndlet-fail ((l locations))
	 (if (evaluate misplaced l)
	     (list 'rearrange l)
	   'fail))))))

(defun make-depots-hierarchy (d)
  (let ((locations (lookup-type d 'locations))
	(pallets (lookup-type d 'pallets))
	(pallet-locs (make-hash-table))
	(s (init-state d)))
    (do-elements (p pallets)
      (do-elements (l locations)
	(when (holds s (list 'at p l))
	  (setf (gethash p pallet-locs) l))))
    
    (create-depots-hierarchy :chains (chains d) :pallet-locs pallet-locs)))



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; helpers
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; needed for goal
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defun crates-needed (src dst cs h)
  "crates-needed SRC DEST COMPLETE-SET HIERARCHY
Return list of crates currently at SRC that need to end up at DEST."
  (let ((needed nil)
	(loc-crates (dcs-loc-crates cs)))
    (do-elements (c (dh-chains h) needed)
      (when (pallet-at (first c) dst h)
	(dolist (crate (rest c))
	  (when (eq (evaluate loc-crates crate) src)
	    (push crate needed)))))))

(defun pallet-at (p loc h)
  (eq (evaluate (dh-pallet-locs h) p) loc))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; misc
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defun locs (h)
  (lookup-type (dh-domain h) 'locations))

|#