(in-package othello)


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Constants
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defparameter *board-symbols* '((empty . #\.) (black . #\X) (white . #\O)))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; functions
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defun othello-io-interface (P1 P2 &key (size 8))
  "othello-io-interface PLAYER1 PLAYER2 &key (SIZE 8).  Simple interface where each player enters moves on the terminal."
  (let ((b (make-init-board size))
	(c 'black)
	(players (list (cons 'black p1) (cons 'white p2))))
    (unless (eq p1 'human) (set-colour 'black p1))
    (unless (eq p2 'human) (set-colour 'white p2))
    (flet ((print-score ()
	     (dolist (p (score b))
	       (format t "~a : ~a    " (car p) (cdr p)))))
	     
      (format t "~&Simple Othello interface.  Enter moves by entering row then column, separated by a space.  Enter nil to quit.")
      (until (game-over? b)
	(fresh-line)
	(pprint-board t b)
	(print-score)
	(let ((move nil)
	      (player (mapping:evaluate players c)))
	  (format t "~%~%~a's turn." (string-capitalize (symbol-name c)))
	  (if (eq player 'human)
	      
	      (until move
		(format t "~%Enter move. ")
		(let* ((r (let ((r (read))) (unless r (return-from othello-io-interface)) r))
		       (col (unless (eq r 'pass) (read)))
		       (l (if (eq r 'pass) 'pass (make-loc r col))))
		  (if (legal? b c l)
		      (setf move l)
		    (format t "~%Illegal move."))))
	    (setf move (make-choice player b)))
	  (format t "~%Move ~a was done." move)
	  (setf b (make-move-on-board b c move)))
	(setf c (opposite-colour c)))
      (format t "~&Game over.  Final position: ")
      (fresh-line)
      (pprint-board t b)
      (print-score))))





(defun pprint-board (str s)

  (dbind (m n) (array-dimensions s)
	 (pprint-logical-block (str nil)
	   (dotimes (i m)
	     (dotimes (j n)
	       (write-char 
		(mapping:evaluate *board-symbols*
				  (aref s i j))
		str))
	     (pprint-newline :mandatory str)))))

(set-pprint-dispatch 'othello-board #'pprint-board)