(in-package othello)


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; class definition
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defclass <othello-env> (<fully-observable-env>)
  ((my-colour :type colour :initarg :my-colour :reader my-colour)
   (opponent :type <othello-policy> :reader opponent :initarg :opponent)
   (board :accessor board))
  (:documentation "<othello-env>

Initargs
:opponent - Type <othello-policy>.
:my-colour - My colour.  Black or white.
"))
  

(defmethod initialize-instance :after ((e <othello-env>) &rest args)
  (declare (ignore args)))
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; create-env methods
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defmethod sample-init ((e <othello-env>))
  (let ((b (make-init-board)))
    (if (eq (my-colour e) 'black)
	b
      (make-opponent-move e b))))

(defmethod sample-next ((e <othello-env>) s a)
  (let* ((c (my-colour e))
	 (b (make-move-on-board s c a t)))
    (make-opponent-move e b)
    (values
     b
     (if (is-terminal-state e b)
	 (let* ((s (score b))
		(score-diff
		 (- (mapping:evaluate s c)
		    (mapping:evaluate s (opposite-colour c)))))
	   (if (> score-diff 0)
	       1
	     (if (< score-diff 0)
		 -1
	       0)))
       0))))
      

(defmethod is-terminal-state ((e <othello-env>) s)
  (game-over? s))

(defmethod avail-actions ((e <othello-env>) s)
  (legal-moves s (my-colour e)))
		      

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; internal
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defun make-opponent-move (e b)
  (let ((opp (opposite-colour (my-colour e))))
    (make-move-on-board b opp (make-choice (opponent e) b) nil)))
