;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; LTK drawing operations for the pick-place environment
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package pick-place)




(defun draw-state (s &aux (e (domain s)))
  (clear-canvas)
  (draw-cspace (cspace s))
  (draw (conf s)))



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; The old code - should be updated
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


(defmethod display-init-state ((e <pick-place-env>) canvas (s <pick-place-state>))
  (let* ((f (cspace-family e))
	 (objects (objects f))
	 (boundary (apply #'create-rectangle canvas (mapcan #'screen-coords (bounds f))))
	 (marker (make-marker objects)))
    (itemconfigure canvas boundary "width" 3)
    
    ;; Draw the robot
    (draw-polygon (world-robot s) canvas :color "darkgreen" :tags "robot")
    (with-slots (held confs held-object-conf) (mode s)
      (let ((conf (sfirst confs)))
	
	;; Draw the marker, which is a small red box centered at vertex 1 of object 0
	(draw-polygon (if (eql 0 held)
		       (transform (conf s) (transform held-object-conf marker))
		     (transform conf marker))
		   canvas :color "red" :tags "marker"))
      
      ;; Draw the movable objects
      (do-elements (conf confs nil i)
	(let ((obj (elt objects i)))
	  (draw-polygon (if (eql i held)
			 (transform (conf s) (transform held-object-conf obj))
		       (transform conf obj))
			canvas :color "darkblue" :tags (object-name i)))))))


    

(defmethod update-state ((e <pick-place-env>) c (s1 <pick-place-state>) (s2 <pick-place-state>))
  ;; Update robot position
  (set-coords c "robot" (poly-coords (world-robot s2)))
  (let ((objects (objects (cspace-family e))))
    (with-slots (held confs held-object-conf) (mode s2)
      
      ;; Update the held object if there is one
      (when held
	(set-coords c (object-name held) 
		    (poly-coords 
		     (transform (conf s2) 
				(transform held-object-conf 
					   (aref objects held))))))
      
      ;; Update the marker if necessary
      (when (eql held 0)
	(set-coords c "marker"
		    (poly-coords 
		     (transform (conf s2) (transform held-object-conf (make-marker objects)))))))))



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Helpers
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defun draw-polygon (poly c &key (color nil csupp) (tags nil tsupp))
  "draw-polygon POLYGON CANVAS &key COLOR TAGS.  

Draws polygon on canvas and returns the resulting polygon item id.  

Any supplied keywords will be set as the corresponding property of the drawn item.  
See TK reference for a list of allowed colors.  
TAGS can be a string of space-separated tags."
  (let ((item (create-polygon c (poly-coords poly))))
    (when csupp
      (itemconfigure c item "fill" color))
    (when tsupp
      (itemconfigure c item "tags" tags))
    item))

(defun poly-coords (p)
  (mapcans #'screen-coords (vertices p)))

(defun object-name (i)
  (format nil "object~a" i))

(defun make-marker (o)
  (let* ((obj (sfirst o))
	 (v (aref (vertices obj) 1)))
    (make-instance '<polygon>
      :vertices (mapcar #'(lambda (d) (a+ d v))
			(list #(-.1 -.1) #(-.1 .1) #(.1 .1) #(.1 -.1))))))
  
  

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Coordinate system
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defparameter *pp-origin-x* 25)
(defparameter *pp-origin-y* 600)
(defvar *pp-zoom* 30)
  
(defun screen-coords (v)
  (dbind (x y) v
    (list (round (+ *pp-origin-x* (* *pp-zoom* x))) (round (- *pp-origin-y* (* *pp-zoom* y))))))

  