;; resource-prog.lisp
;; the Alisp partial program for the resource collection environment
;; adapted from Oregon State MAXQ TL team's original design 
;; should be used with resource-env.lisp, which defines the resource collection 
;; stratagus environment

(defpackage resource-prog
 (:documentation "resource-prog.lisp - Alisp program for resource-env.lisp domain
Functions
---------
resource-prog")

 (:use resource-env
       stratagus-env
       common-lisp
       utils
       calisp-prog)
 (:import-from calisp js-env-state threads-at-label js-thread-states ts-label)
 (:export <resource-prog>))

(in-package resource-prog)

; CONSTANTS - refers to implemented map sizes
(defconstant *map-length* 32)
(defconstant *map-width* 32)

; single-thread-chooser enforces the rule that threads choose one at a time; this is necessary for 
; computing per-peasant rewards in resource-env 
(defclass <resource-prog> (<calisp-program>)
  ((choosing-thread-fn :initform #'single-thread-chooser)))

; root thread spawns threads for individual peasants, then goes to sleep on no_op actions
(defmethod start ((p <resource-prog>))
  (dolist (i (units-belonging-to-of-type 0 2 (env-state))) ; 0 -> red player (0); 2 -> peasant
    (spawn (cons 'peasant i) Root () i))
  (loop
   (action no_ops 0)))

; peasant root subtask, chooses (until episode terminates) between getting gold, getting wood,
; and depositing what was gotten
(defun Root ()
  (loop
    (choose Root
      (Get_Gold (Get_Gold))
      (Get_Wood (Get_Wood))
      (Deposit  (Deposit)))))

; old version
; (defun Get-Gold ()
;  (until (equalp '(1) (unit-status-args (get-unit-state (car (my-effectors)) (env-state))))
;    (with-choice harvest-choice (ct (list 0 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16)) ; choice - 0-15 regions, 16 harvest gold action
;      (if (= ct 16)
;	(progn
;	  (action Harvest-Gold (format nil "6 1"))
;	  (until (= 1 (unit-status (get-unit-state (car (my-effectors)) (env-state))))
;           (action no-ops 0)))
;	(Goto ct)))))

; get_gold: peasant subtask for getting gold
; termination condition: peasant has gold (unit status arg = (1))
; choices: goto(k), where k is one of the regions of the map, or harvest_gold, which harvests from
; a gold mine in range

; note that actions like goto(k) and harvest_gold might take more than one stratagus env step to complete; no_ops are executed
; till then

(defun Get_Gold ()
  (until (equalp '(1) (unit-status-args (get-unit-state (car (my-effectors)) (env-state))))
	(choose get_gold
		(goto0 (goto0))
		(goto1 (goto1))
		(goto2 (goto2))
		(goto3 (goto3))
		(goto4 (goto4))
		(goto5 (goto5))
		(goto6 (goto6))
		(goto7 (goto7))
		(goto8 (goto8))
		(goto9 (goto9))
		(goto10 (goto10))
		(goto11 (goto11))
		(goto12 (goto12))
		(goto13 (goto13))
		(goto14 (goto14))
		(goto15 (goto15))
		(harvest_gold (harvest_gold)))))

(defun harvest_gold ()
  (action Harvest_Gold (format nil "6 1"))
    (until (= 1 (unit-status (get-unit-state (car (my-effectors)) (env-state))))
      (action no_ops 0)))

; get_wood: peasant subtask for getting wood
; termination condition: peasant has wood (unit status arg = (2))
; choices: goto(k), where k is one of the regions of the map, or harvest_wood, which harvests from
; a tree in range

(defun Get_Wood ()
  (until (equalp '(2) (unit-status-args (get-unit-state (car (my-effectors)) (env-state))))
	(choose get_wood
		(goto0 (goto0))
		(goto1 (goto1))
		(goto2 (goto2))
		(goto3 (goto3))
		(goto4 (goto4))
		(goto5 (goto5))
		(goto6 (goto6))
		(goto7 (goto7))
		(goto8 (goto8))
		(goto9 (goto9))
		(goto10 (goto10))
		(goto11 (goto11))
		(goto12 (goto12))
		(goto13 (goto13))
		(goto14 (goto14))
		(goto15 (goto15))
		(harvest_wood (harvest_wood)))))

(defun harvest_wood ()
  (action Harvest_Wood (format nil "6 2"))
    (until (= 1 (unit-status (get-unit-state (car (my-effectors)) (env-state))))
      (action no_ops 0)))

; deposit: peasant subtask for depositing held resources, if any
; termination condition: peasant has nothing (unit status arg = nil)
; choices: goto(k), where k is one of the regions of the map, or dropoff, which drops whatever is carried
; to a townhall in range

; note that if called while not carrying any resources, will return immediately due to the until clause

(defun Deposit ()
    (until (null (unit-status-args (get-unit-state (car (my-effectors)) (env-state))))
	(choose deposit
		(goto0 (goto0))
		(goto1 (goto1))
		(goto2 (goto2))
		(goto3 (goto3))
		(goto4 (goto4))
		(goto5 (goto5))
		(goto6 (goto6))
		(goto7 (goto7))
		(goto8 (goto8))
		(goto9 (goto9))
		(goto10 (goto10))
		(goto11 (goto11))
		(goto12 (goto12))
		(goto13 (goto13))
		(goto14 (goto14))
		(goto15 (goto15))
		(dropoff (dropoff)))))

(defun dropoff ()	
  (progn
    (action dropoff (format nil "7"))
    (until (= 1 (unit-status (get-unit-state (car (my-effectors)) (env-state))))
      (action no_ops 0))))

; old version
;(defun Get-Wood ()
;  (until (equalp '(2) (unit-status-args (get-unit-state (car (my-effectors)) (env-state))))
;    (with-choice harvest-choice (ct (list 0 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16)) ; choice - 0-15 regions, 16 harvest wood action
;      (if (= ct 16)	 
;	(progn
;	  (action Harvest-Wood (format nil "6 2"))
;	  (until (= 1 (unit-status (get-unit-state (car (my-effectors)) (env-state))))
;           (action no-ops 0)))
;	(Goto ct)))))

; old version
;(defun Goto (k)
;  (action (cons goto k) (format nil "2 ~a ~a" (car (region-to-coordinate k)) (cdr (region-to-coordinate k))))
;  (until (= 1 (unit-status (get-unit-state (car (my-effectors)) (env-state))))
;    (action no-ops 0)))

; the goto(k) subtask is adapted into 16 different functions, one for each region; this makes it easier to 
; differentiate them as thread labels

(defun goto0 ()
  (action goto0 (format nil "2 ~a ~a" (car (region-to-coordinate 1)) (cdr (region-to-coordinate 0))))
  (until (= 1 (unit-status (get-unit-state (car (my-effectors)) (env-state))))
	(action no_ops 0)))

(defun goto1 ()
  (action goto1 (format nil "2 ~a ~a" (car (region-to-coordinate 1)) (cdr (region-to-coordinate 1))))
  (until (= 1 (unit-status (get-unit-state (car (my-effectors)) (env-state))))
	(action no_ops 0)))

(defun goto2 ()
  (action goto2 (format nil "2 ~a ~a" (car (region-to-coordinate 2)) (cdr (region-to-coordinate 2))))
  (until (= 1 (unit-status (get-unit-state (car (my-effectors)) (env-state))))
	(action no_ops 0)))

(defun goto3 ()
  (action goto3 (format nil "2 ~a ~a" (car (region-to-coordinate 3)) (cdr (region-to-coordinate 3))))
  (until (= 1 (unit-status (get-unit-state (car (my-effectors)) (env-state))))
	(action no_ops 0)))

(defun goto4 ()
  (action goto4 (format nil "2 ~a ~a" (car (region-to-coordinate 4)) (cdr (region-to-coordinate 4))))
  (until (= 1 (unit-status (get-unit-state (car (my-effectors)) (env-state))))
	(action no_ops 0)))

(defun goto5 ()
  (action goto5 (format nil "2 ~a ~a" (car (region-to-coordinate 5)) (cdr (region-to-coordinate 5))))
  (until (= 1 (unit-status (get-unit-state (car (my-effectors)) (env-state))))
	(action no_ops 0)))

(defun goto6 ()
  (action goto6 (format nil "2 ~a ~a" (car (region-to-coordinate 6)) (cdr (region-to-coordinate 6))))
  (until (= 1 (unit-status (get-unit-state (car (my-effectors)) (env-state))))
	(action no_ops 0)))

(defun goto7 ()
  (action goto7 (format nil "2 ~a ~a" (car (region-to-coordinate 7)) (cdr (region-to-coordinate 7))))
  (until (= 1 (unit-status (get-unit-state (car (my-effectors)) (env-state))))
	(action no_ops 0)))

(defun goto8 ()
  (action goto8 (format nil "2 ~a ~a" (car (region-to-coordinate 8)) (cdr (region-to-coordinate 8))))
  (until (= 1 (unit-status (get-unit-state (car (my-effectors)) (env-state))))
	(action no_ops 0)))

(defun goto9 ()
  (action goto9 (format nil "2 ~a ~a" (car (region-to-coordinate 9)) (cdr (region-to-coordinate 9))))
  (until (= 1 (unit-status (get-unit-state (car (my-effectors)) (env-state))))
	(action no_ops 0)))

(defun goto10 ()
  (action goto10 (format nil "2 ~a ~a" (car (region-to-coordinate 10)) (cdr (region-to-coordinate 10))))
  (until (= 1 (unit-status (get-unit-state (car (my-effectors)) (env-state))))
	(action no_ops 0)))

(defun goto11 ()
  (action goto11 (format nil "2 ~a ~a" (car (region-to-coordinate 11)) (cdr (region-to-coordinate 11))))
  (until (= 1 (unit-status (get-unit-state (car (my-effectors)) (env-state))))
	(action no_ops 0)))

(defun goto12 ()
  (action goto12 (format nil "2 ~a ~a" (car (region-to-coordinate 12)) (cdr (region-to-coordinate 12))))
  (until (= 1 (unit-status (get-unit-state (car (my-effectors)) (env-state))))
	(action no_ops 0)))

(defun goto13 ()
  (action goto13 (format nil "2 ~a ~a" (car (region-to-coordinate 13)) (cdr (region-to-coordinate 13))))
  (until (= 1 (unit-status (get-unit-state (car (my-effectors)) (env-state))))
	(action no_ops 0)))

(defun goto14 ()
  (action goto14 (format nil "2 ~a ~a" (car (region-to-coordinate 14)) (cdr (region-to-coordinate 14))))
  (until (= 1 (unit-status (get-unit-state (car (my-effectors)) (env-state))))
	(action no_ops 0)))

(defun goto15 ()
  (action goto15 (format nil "2 ~a ~a" (car (region-to-coordinate 15)) (cdr (region-to-coordinate 15))))
  (until (= 1 (unit-status (get-unit-state (car (my-effectors)) (env-state))))
	(action no_ops 0)))

; old version
; (defun Deposit ()
;    (until (null (unit-status-args (get-unit-state (car (my-effectors)) (env-state))))
;    (with-choice dropoff-choice (ct (list 0 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16)) ; choice - 0-15 regions, 16 dropoff action
;      (if (= ct 16)
;	(progn
;	  (action dropoff (format nil "7"))
;	  (until (= 1 (unit-status (get-unit-state (car (my-effectors)) (env-state))))
;           (action no-ops 0)))
;	(Goto ct)))))

; converts a region (from 0-15) to a coordinate that is at the center of that region
; works only with 32x32 maps divided into 16 regions (for details see the Oregon State power point
; presentation)
(defun region-to-coordinate (k)
  (if (and (>= k 0) (<= k 8))
    (cons (+ 4 (* (mod k 3) 9)) (+ 4 (* (floor (/ k 3)) 9)))
    (if (and (>= k 9) (<= k 11))
	(cons (+ 4 (* (mod k 3) 9)) 29)
        (if (and (>= k 12) (<= k 14))
	    (cons 29 (+ 4 (* (mod k 3) 9)))
	    (if (= k 15)
		(cons 29 29)
	        'error)))))

;; choosing-thread-fn that allows exactly one thread to choose at a time

(defun single-thread-chooser (omega)
  ;(print "THREAD STATES: ")
  ;(print (calisp:js-thread-states omega))

  (pick-random (calisp::get-choosing-threads 'all omega)))

;  (hash-table-select 
;   (calisp:js-thread-states omega)
;   (lambda (v) (eq (calisp:ts-status v) 'holding))))

;; pick-random returns a random member of a list in a list

(defun pick-random (lst)
  (if (<= (list-length lst) 0) nil
    (list (nth (random (list-length lst)) lst))))
    
;; subtask hash table for this partial program
;; according to the subtask hierarchy given by the Oregon State power point presentation
(setf subtask-table (make-hash-table))

(setf (gethash 'root subtask-table)     (list 'get_gold 'get_wood 'deposit))
(setf (gethash 'get_gold subtask-table) (list 'harvest_gold 'goto0 'goto1 'goto2 'goto3 'goto4 'goto5 'goto6 'goto7 'goto8 'goto9 'goto10 'goto11 'goto12 'goto13 'goto14 'goto15))
(setf (gethash 'get_wood subtask-table) (list 'harvest_wood 'goto0 'goto1 'goto2 'goto3 'goto4 'goto5 'goto6 'goto7 'goto8 'goto9 'goto10 'goto11 'goto12 'goto13 'goto14 'goto15))
(setf (gethash 'deposit subtask-table)  (list 'dropoff 'goto0 'goto1 'goto2 'goto3 'goto4 'goto5 'goto6 'goto7 'goto8 'goto9 'goto10 'goto11 'goto12 'goto13 'goto14 'goto15))

; we assume that rewards are already distributed by peasant in an assoc list
; all we need to do then is to make them distributed by threads
; since peasants map to threads on a one-to-one basis, this is easy

(defun distribute-reward (omega prev act rew next)
  (let ((rew-list '()))
    (dolist (p rew)
      (setf rew-list (acons (cons 'peasant (car p)) (cdr p) rew-list)))
    rew-list))

; given an effector and omega, gives the thread id for the thread that is assigned that effector
; we'll take the short-cut -- since we know how the thread ids are made
(defun get-thread-id-from-effector (omega effector)
  (cons 'peasant effector))