(in-package stratagus-env)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; the overall env state
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defstruct (strat-env-state (:conc-name strat-es-) (:constructor make-strat-es))
  "Structure strat-env-state.  
Constructor : make-strat-es
Fields :
global - Accessed using strat-es-global.  A structure of type global-state
units - Accessed using strat-es-units.  An array mapping unit id to unit structures"
  global
  units)

(defparameter *not-necessary-to-print* '(58))

(defun pprint-strat-env-state (&rest args)
  (bind-pprint-args (str s) args
    (if *print-readably*
	(write s :stream str :pretty nil)
      (pprint-logical-block (str nil :prefix "[" :suffix "]")
	(format str "Strat env state~2I~:@_Global state : ~w~:@_Unit states:~4I" (strat-es-global s))
	(do-elements (u (strat-es-units s) nil i)
	  (when u
	    (unless (member (unit-type u) *not-necessary-to-print*)
	      (pprint-pop)
	      (format str "~:@_~a : ~w" i u))))))))

(set-pprint-dispatch 'strat-env-state #'pprint-strat-env-state)

(defun unit-id-ceiling (s)
  (length (strat-es-units s)))

(defun get-unit-state (id s)
  "get-unit-state ID S.  S is of type strat-env-state.  Returns a structure of type unit, or nil if unit doesn't exist."
  (aref (strat-es-units s) id))

(defun units-belonging-to (id s)
  "units-belonging-to PLAYER-ID STRAT-ENV-STATE.  Return list of ids of units belonging to this player.  Only returns those units with HP > 0, to deal with the ghost units that stick around for a while after they die."
  (let ((l nil))
    (do-elements (us (strat-es-units s) l i)
      (when (and (= (unit-player-id us) id) (> (unit-hp us) 0))
	(push i l)))))

(defun units-belonging-to-of-type (id type s)
  "units-belonging-to-of-type PLAYER-ID STRAT-ENV-STATE."
  (let ((l nil))
    (do-elements (us (strat-es-units s) l i)
      (when (and us (= (unit-player-id us) id) (> (unit-hp us) 0)
		 (eql (unit-type us) type))
	(push i l)))))

(defun get-unit-loc (id s)
  (unit-loc (get-unit-state id s)))

(defun get-unit-status (id s)
  (lookup-status (unit-status (get-unit-state id s))))

(defun all-units (s)
  "Returns ALL unit IDs on the map with hp > 0"
  (filter ':list (strat-es-units s)
	  #'(lambda (us) (or (= (unit-type us) 93) (> (unit-hp us) 0)))))

(defmethod effectors ((s strat-env-state))
  (nreverse (units-belonging-to *my-player-id* s)))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; global-state
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defstruct global-state
  (gold :type fixnum)
  (wood :type fixnum))


(defmethod clone ((g global-state))
  (copy-structure g))

(defun gold (s)
  (global-state-gold (strat-es-global s)))

(defun wood (s)
  (global-state-wood (strat-es-global s)))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; unit states
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defstruct unit
  player-id
  type
  loc
  hp
  r-amt
  status
  status-args
  kills)


;; clone
;; u : unit
;;
;; returns a fresh structure with all the fields copied.  
(defmethod clone ((u unit))
  (make-unit
   :player-id (unit-player-id u)
   :type (unit-type u)
   :loc (copy-list (unit-loc u))
   :hp (unit-hp u)
   :r-amt (unit-r-amt u)
   :status (unit-status u)
   :status-args (copy-list (unit-status-args u))
   :kills (unit-kills u)))
  



(in-package cl-user)
