(in-package asdf)
			 

(asdf:defsystem "hrl"
    :components ((:module "misc" :pathname "misc/" :serial t
			  :components
			  ((:file "util-pkg")
			   (:file "macros")
			   (:file "clone")
			   (:file "canonicalize" :depends-on ("clone"))
			   (:file "cluster")
			   (:file "help")
			   (:file "utils" :depends-on ("clone" "util-pkg"))
			   (:file "array-utils" :depends-on ("utils" "macros"))
			   (:file "list-utils" :depends-on ("utils"))
			   (:file "hash-utils" :depends-on ("utils"))
			   (:file "function-utils")
			   (:file "exp-utils")
			   (:file "math-utils" :depends-on ("utils"))
			   (:file "matlab" :depends-on ("utils"))
			   (:file "sequence-utils" :depends-on ("utils"))
			   (:file "string-utils" :depends-on ("sequence-utils"))
			   ))
		 
		 
		 
		 (:module "set" :pathname "data-struct/set/"
			  :depends-on ("misc")
			  :components ((:file "set")
				       (:file "seq-set" :depends-on ("set"))
				       (:file "hash-set" :depends-on ("set"))
				       (:file "number-set" :depends-on ("set"))
				       (:file "indexed-set" :depends-on ("set"))
				       (:file "recursive-enumeration" :depends-on ("set"))
				       (:file "named-sets" :depends-on ("set"))
				       (:file "inst-var-accessors" :depends-on ("set"))
				       (:file "create-sets" :depends-on ("set" "indexed-set" "inst-var-accessors"))
				       (:file "interval" :depends-on ("set"))
				       (:file "directory-set" :depends-on ("set"))
				       (:file "direct-product-set" :depends-on ("set" "inst-var-accessors"))))
		 
		 (:module "mapping" :pathname "data-struct/"
			  :depends-on ("misc" "set")
			  :components
			  ((:file "mapping")))

		 (:module "data-struct" :pathname "data-struct/"
			  :depends-on ("misc" "prob" "set" "mapping")
			  :components
			  ((:file "bucketed-counts")
			   (:file "circular-vector")
			   (:file "local-search")
			   (:file "queue")
			   (:file "bst" :depends-on ("tree"))
			   (:file "priority-queue")
			   (:module "graph"
				    :depends-on ("queue" "priority-queue" "union-find")
				    :components
				    ((:file "graph")
				     (:file "topological-sort" :depends-on ("graph"))
				     (:file "adjacency-list-graph" :depends-on ("graph"))
				     (:file "adjacency-matrix-graph" :depends-on ("graph"))
				     (:file "algorithms" :depends-on ("graph"))
				     (:file "connected-components" :depends-on ("graph"))
				     (:file "flow" :depends-on ("adjacency-list-graph"))))
			   (:module "bnet"
				    :depends-on ("graph")
				    :components ((:file "bnet-pkg")
						 (:file "2tbn" :depends-on ("bnet-pkg"))
						 (:file "structure-learning" :depends-on ("2tbn"))))
			   (:file "skiplist")
			   (:module "tree"
				    :components ((:file "tree")
						 (:file "tree-set" :depends-on ("tree"))))
			   
			   (:file "union-find" :depends-on ("tree"))
			   
			   (:module "prop-logic" :depends-on ("tree")
				    :components ((:file "prop-formula")
						 (:file "dnf" :depends-on ("prop-formula"))
						 (:file "dnf-set" :depends-on ("dnf"))))
			   
 			   (:module "pot-set"
 				    :components ((:file "pot-pkg")
						 (:file "operation" :depends-on ("pot-pkg"))
 						 (:file "potential" :depends-on ("pot-pkg"))
 						 (:file "tabular-potential" :depends-on ("potential"))
 						 (:file "function-potential" :depends-on ("potential"))
						 (:file "number-potential" :depends-on ("potential"))
					 	 (:file "pot-set" :depends-on ("potential" "operation"))
						 (:file "undirected-graphical-model" :depends-on ("pot-set"))
						 ))

			   (:file "cache")
			   ))
		 
		 
		 (:module "env" :depends-on ("misc" "set" "prob")
			  :components ((:file "env-pkg")
				       (:file "env" :depends-on ("env-pkg"))
				       (:file "fully-observable-env" :depends-on ("env"))
				       (:file "trajectory" :depends-on ("env"))
				       ))
		 
		 (:module "fn-approx" :pathname "fn-approx/"
			  :depends-on ("misc")
			  :components ((:file "fn-approx")
				       (:file "tabular-fn-approx" :depends-on ("fn-approx"))
				       (:file "linear-fn-approx" :depends-on ("fn-approx"))
				       (:file "linear-fn-approx-with-bounds" :depends-on ("fn-approx"))))
		 
		 (:module "rl" :depends-on ("mdp" "data-struct" "misc" "env" "rl-functions" "dp")
			  :components ((:file "reinforcement-learning")
				       (:file "rl-observer" :depends-on ("reinforcement-learning"))
				       (:file "rl-control" :depends-on ("rl-observer" "reinforcement-learning"))
				       (:file "rl-user" :depends-on ("reinforcement-learning" "rl-observer" 
											      "rl-control" "obs"))
				       (:module "obs"
						:depends-on ("rl-observer")
						:components ((:file "progress-printer")
							     (:file "stat-gatherer")
							     (:file "message-logger")
							     (:file "trajectory-gatherer")
							     (:file "env-observer")))
				       (:module "learn"
						:depends-on ("obs")
						:components ((:file "learning-algorithm")
							     (:file "q-learning" :depends-on ("learning-algorithm"))
							     (:file "advantage-updating" :depends-on ("learning-algorithm"))
							     (:file "decomposed-advantage-updating"
								    :depends-on ("advantage-updating"))
							     (:file "decomposed-q-learning" :depends-on
								    ("q-learning"))
							     (:file "approximate-policy-iteration"
								    :depends-on ("learning-algorithm"))
							     (:file "gold-standard" :depends-on ("learning-algorithm"))
							     (:file "certainty-equivalence" :depends-on ("learning-algorithm"))
							     ))))

		 (:module "rl-functions"
			  :depends-on ("fn-approx" "misc" "data-struct" "prob" "math" "mdp")
			  :components ((:module "value-function" :depends-on ("policy")
						:components ((:file "value-function")
							     (:file "tabular-value-function" :depends-on
								    ("value-function"))))
				       
						
				       (:module "q-function"
						:depends-on ("value-function" "policy")
						:components ((:file "q-function")
 							     (:module "crl" :depends-on ("q-function")
 								      :components
 								      ((:file "crl-q-function")
 								       (:file "crl-features"
 									      :depends-on 
									      ("crl-q-function"))))
							     (:file "sum-q-function" 
								    :depends-on ("q-function"))
							     (:file "approx-q-function" :depends-on 
								    ("q-function"))
							     (:file "tabular-q-function" :depends-on
								    ("q-function"))
							     (:module "decomposed" :depends-on ("crl")
								      :components
								      ((:file "decomposed-q-function")
								       (:file "decomposed-crl-q-function")
								       (:file "decomposed-tabular-q-function")))
							     (:file "env-q-function" :depends-on 
								    ("approx-q-function"))
							     ))
				       
				       (:module "policy"
						:components ((:file "policy")
							     
							     (:file "random-policy" :depends-on ("policy"))
							     (:file "tabular-policy" :depends-on ("policy"))
							     (:file "prompt-policy" :depends-on ("policy"))
							     ))
				       
				       (:module "policy2" :depends-on ("q-function" "policy")
						:pathname "policy/"
						:components ((:file "greedy-policy")
							     (:module "exp-pol"
								      :components
								      ((:file "exploration-policy")))))
				       
				       (:module "advantage-function" :depends-on ("value-function" "policy")
						:components ((:file "advantage-function")
							     (:file "tabular-advantage-function"
								    :depends-on ("advantage-function"))
							     (:file "sum-advantage-function"
								    :depends-on ("advantage-function"))))


				       (:module "learning-rate"
						:components ((:file "learning-rate")
							     (:file "polynomial-learning-rate" 
								    :depends-on ("learning-rate"))))))
		 
		 (:module "math" :pathname "math/" :depends-on ("misc" "set" )
			  :components ((:file "lin-alg")
				       (:file "chi-square")
				       (:file "svd" :depends-on ("lin-alg"))
				       ))
		 
		 
		 
		 
		 
		 (:module "prob"
			  :depends-on ("set" "mapping" "math")
			  :components ((:file "probability-distribution")
				       (:file "function-random-variable" :depends-on ("probability-distribution"))
				       (:file "information-theory" :depends-on ("probability-distribution"))
				       (:file "create-distributions" :depends-on ("probability-distribution"))
				       (:file "vector-probability-distribution" :depends-on ("probability-distribution"))
				       (:file "hash-table-prob-dist" :depends-on ("probability-distribution"))
				       (:module "temporal"
						:components ((:file "state-space-model")
							     )
						:depends-on ("probability-distribution"))
				       (:module "parametric"
						:depends-on ("probability-distribution")
						:components ((:file "uniform")
							     (:file "gaussian")))
				       (:module "filtering"
						:depends-on ("probability-distribution" "temporal")
						:components					
						((:file "filtering")
						 (:module "dmcmc"
							  :depends-on ("filtering" "exact")
							  :components
							  ((:file "dmcmc")
							   (:file "dmcmc-filter" :depends-on ("dmcmc" "decay"))
							   (:file "decay" :depends-on ("dmcmc"))
							   (:file "run-dmc" :depends-on ("dmcmc-filter"))
							   (:file "observer" :depends-on ("dmcmc"))))
						 (:module "pf"
							  :depends-on ("filtering")
							  :components 
							  ((:file "pf")))
						 (:module "exact"
							  :depends-on ("filtering")
							  :components
							  ((:file "exact")))
						 (:module "exp"
							  :depends-on ("dmcmc" "exact" "pf")
							  :components
							  ((:file "dmcmc-scaling")))))
				       (:file "alist-probability-distribution" :depends-on ("probability-distribution"))))
		 
		      
		 (:module "mdp" :depends-on ("env" "data-struct" "math" "prob" "misc")
			  :in-order-to ((compile-op (load-op "misc")))						   
			  :components ((:file "mdp-pkg")
				       (:file "smdp"
					      :depends-on ("mdp-pkg"))
				       (:file "mdp"
					      :depends-on ("smdp"))
				       (:file "mdp-env"
					      :depends-on ("mdp"))
				       (:file "2tbn-mdp-env"
					      :depends-on ("mdp-env"))
				       (:file "tabular-smdp"
					      :depends-on ("smdp"))
				       (:file "hierarchical-smdp"
					      :depends-on ("smdp" "tabular-smdp"))
				       (:file "tabular-mdp" :depends-on ("mdp"))
				       (:file "sparse-mdp" :depends-on ("mdp"))))
     
		 (:module "dp" :depends-on ("mdp" "data-struct" "misc" "math" "rl-functions")
			  :components ((:file "dp")
				       (:file "mdp-dp" :depends-on ("dp"))
				       (:file "sparse-dp" :depends-on ("mdp-dp"))
				       (:file "hierarchical-dp" :depends-on ("dp" "sparse-dp"))
				       (:file "markov-chain" :depends-on ("dp"))
				       ))
     
		 


		 
		 (:module "envs" :depends-on ("misc" "data-struct" "mdp" "math")
			  :components ((:file "grid-world")
				       (:file "maze-mdp" :depends-on ("grid-world"))
				       (:file "variable-effector-env")
				       (:module "res-balance" :depends-on ("grid-world")
						:components ((:file "rbe-2tbn")))
				       (:module "flags" :depends-on ("grid-world")
						:components ((:file "flags")))
				       (:module "othello"
						:components ((:file "othello")
							     (:file "policies" :depends-on ("othello"))
							     (:file "interface" :depends-on ("othello"))
							     (:file "othello-env" :depends-on ("othello"))
							     (:file "features" :depends-on ("othello"))))
				       (:module "taxi"
						:depends-on ("grid-world")
						:components ((:file "td-taxi-env")
							     (:file "td-taxi-examples")
							     (:file "td-taxi-flat-lfa" :depends-on ("td-taxi-env"))
							     (:file "qe-taxi")
							     ))))
     


		 (:module "test" :depends-on ("envs" "misc")
			  :components ((:file "mdp-test-envs")))))
	









(in-package cl-user)




;; Local variables:
;; mode:lisp
;; outline-regexp:"\\s-*..module"
;; End:
