(defpackage test-nav-switch
  (:use
   cl
   utils
   set
   hla
   nav-switch
   prop-logic
   ))

(in-package test-nav-switch)

(defvars 
    c d h p q g)


(setf c (make-hash-subsumption-checker #'subsumption-key)
      )
    
(defun generate-random-world (nr nc ns)
  (random-ns-world nr nc ns 2 6))

(defun use-world (d2)
  (setf
      d d2
      h (make-flat-prop-hierarchy d2)
      p (make-instance '<prop-abstract-planning-problem>
	  :preprocess t :admissible-heuristic #'ns-act-complete :planning-problem d2 :hierarchy h
	  :subsumption-checker c)
      g (nav-switch-hierarchy d2)
      q (make-instance '<prop-abstract-planning-problem>
	  :preprocess t :admissible-heuristic #'ns-act-complete :planning-problem d2 :hierarchy g
	  :guarantee-heuristic #'ns-act-sound :complete-desc-schemas (ns-descs d2)
	  :sound-desc-schemas (ns-descs d2) :subsumption-checker c)))


(defun complete-reward (x y z)
  (declare (ignore x z))
  y)



(defun offline-tests ()

  (flet ((check (p)
	   (assert (hla:succeeds? d (subseq p 0 (1- (length p)))))))
    (format t "~&------------------------------------------------~&AHSS~&")
    (force-output t)
    (time (check (ahss q '-infty)))
    (format t "~&------------------------------------------------~&AHA*~&")
    (force-output t)
    (time (check (aha* q)))
    (format t "~&------------------------------------------------~&Flat A*~&")
    (force-output t)
    (time (check (aha* p)))
    (force-output t)))



(defparameter *default-max* 1000)

(defun htest (nr)
  (time (print (rl:evaluate d (ahrta*-agent q #'complete-reward nr :max-num-actions 5 ) :num-steps *default-max*))))

(defun ftest (nr)
  (time (print (rl:evaluate d (ahrta*-agent p #'complete-reward nr :max-num-actions 5 ) :num-steps *default-max*))))

(defparameter *nrs* '(100 200 300 400 500 600 700 800 900 1000))

(defun online-tests (&optional (nrs *nrs*))
  (dolist (nr nrs)
    (format t "~&-----------------------------------------------~&AHLRTA*~&~a refinements per step~&" nr)
    (htest nr)
    (format t "~&-----------------------------------------------~&LRTA*~&~a refinements per step~&" nr)
    (ftest nr)

    ))



(defvar *doms*)

(setf *doms*
  (vector
   (make-nav-switch-world 5 5 '((1 1) (2 3) (3 2)) '(3 3) '(0 1))
   (make-nav-switch-world 10 10 '((1 6) (8 2) (4 7)) '(7 7) '(2 2))
   (make-nav-switch-world 20 20 '((2 12) (15 2) (7 17)) '(15 15) '(5 5))
   (make-nav-switch-world 40 40 '((3 29) (33 11) (23 20)) '(35 35) '(2 3))
   (make-nav-switch-world 50 50 '((6 6) (23 44) (34 23)) '(40 40) '(3 9))
   (make-nav-switch-world 60 60 '((5 23) (35 44) (41 6)) '(50 50) '(7 4))))
   

(setf *debug-level* 1)

(defun run-online (d &optional (nrs *nrs*))
  (use-world (aref *doms* d))
  (online-tests nrs))

(defun run-offline (d)
  (use-world (aref *doms* d))
  (offline-tests))
  