(defpackage test-ww
  (:use lookahead
	utils
	prob
	tree
	blocks
	cl
	mapping
	env-user
	set
	prop-logic))


(in-package test-ww)


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; examples
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


(defvars d u f q c g h)

(setf 
 *debug-level* 2
    d (make-array 100)
    u (make-array 100)
    f (make-array 100)
    q (make-array 100)
    c (make-array 100)
    g (make-array 100)
    h (make-array 100)
    (aref d 5) (make-blocks-world-with-ceiling 8 4 '((a 0 1) (baz 2 1) (c 2 2))
					       '(2 3) '(and (block-pos a 1 2) (block-pos c 1 1)))
    (aref d 1) (make-blocks-world-with-ceiling 4 3 '((a 0 1) (baz 2 1) (c 2 2))
				       '(2 3) '(and (on c a)))
    (aref d 2) (make-blocks-world-with-ceiling 4 4 '((a 0 1) (baz 2 1) (c 2 2))
				       '(2 3) '(and (on c a) (on baz c)))

    (aref d 4) (make-blocks-world-with-ceiling 8 4 '((a 0 1) (baz 2 1) (c 2 2))
				       '(2 4) '(and (on baz :t3) (on c baz)))
    (aref d 6) (make-blocks-world-with-ceiling 6 4 '((a 0 1) (baz 2 1) (c 2 2))
				       '(2 4) '(and (on baz :t3) (on c baz)))
    (aref d 3) (make-blocks-world-with-ceiling 4 4 '((a 0 1) (baz 2 1) (c 2 2))
				       '(2 3) '(and (on c :t3) (on baz c) (on a baz)))
    )

(for-loop (i 1 7)
	  (setf
	      (aref c i) (make-blocks-subsumption-checker (aref d i))
	      (aref f i) (make-instance '<prop-abstract-planning-problem>
			   :preprocess t :admissible-heuristic (dist-heuristic (aref d i))
			   :subsumption-checker (aref c i)
			   :planning-problem (aref d i) :hierarchy (make-flat-prop-hierarchy (aref d i)))
	      (aref u i) (make-unguided-blocks-hierarchy (aref d i))
	      (aref q i) (make-instance '<prop-abstract-planning-problem>
			   :subsumption-checker (aref c i)
			   :preprocess t :admissible-heuristic (dist-heuristic (aref d i))
			   :planning-problem (aref d i) :hierarchy (aref u i)
			   :complete-desc-schemas (make-complete-descriptions (aref d i))
			   :sound-desc-schemas (make-sound-descriptions (aref d i)))
	      (aref h i) (make-navigateless-hierarchy (aref d i))
	      (aref g i) (make-instance '<prop-abstract-planning-problem>
			   :preprocess t :admissible-heuristic (dist-heuristic (aref d i))
			   :subsumption-checker (aref c i)
			   :planning-problem (aref d i) :hierarchy (aref h i)
			   :complete-desc-schemas (make-complete-descriptions (aref d i))
			   :sound-desc-schemas (make-sound-descriptions (aref d i)))))


	  



	

(defun priority-fn (a cr sr)
  (mymax
   (my* (my+ cr sr) .5)
   (if (eq a 'act)
       (mymin (my* cr 3) -1)
     (my* cr 1.5))))

(defun complete-reward (a cr sr)
  (declare (ignore a sr))
  cr)

(compile 'complete-reward)
(compile 'priority-fn)

(defparameter *default-max* 10000)
(defvar *io-stream* nil)
(defparameter *nrs* '(100 200 300 400 500 600 700 800 900 1000 1500 2000 2500 3000 3500 4000 4500 5000))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; online tests
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;



(defun gtest (i nr &optional (num-steps *default-max*))
  "ahrta* with complete value*"
  (time
   (print
   (rl-user:evaluate 
    (aref d i)
    (hla::ahrta*-agent (aref g i) #'complete-reward nr  :max-num-actions 6
		       :refinement-ind-choice ':sound-complete-gap)
    :env-display-stream *io-stream*
    :num-steps num-steps))))

(defun ftest (i nr &optional (num-steps *default-max*))
  "flat"
  (time
   (print
   (rl-user:evaluate 
    (aref d i)
    (hla::ahrta*-agent (aref f i) #'complete-reward nr :max-num-actions 6)
    :env-display-stream *io-stream*
    :num-steps num-steps))))



(setf *debug-level* 1)
(defun run-online (i &optional (nrs *nrs*))
  (dolist (nr nrs)
    (format t "~&-----------------------------------------------~&AHLRTA*~&~a refinements per step~&" nr)
    (gtest i nr)
    (format t "~&-----------------------------------------------~&LRTA*~&~a refinements per step~&" nr)
    (ftest i nr)))


(defun ptest (i nr &optional (num-steps *default-max*))
  "ahrta* with priorities"
  (time
   (print
    (rl-user:evaluate 
     (aref d i)
     (hla::ahrta*-agent (aref g i) #'priority-fn nr :max-num-actions 6
			:refinement-ind-choice ':sound-complete-gap :final-choice ':highest-h)
     :env-display-stream *io-stream*
     :num-steps num-steps))))
 




;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; offline exps
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defun aha (i)
  (time
   (aha* (aref q i) :termination ':primitive)))

(defun ahs (i)
  (time 
   (ahss (aref q i) '-infty :priority-fn #'priority-fn 
	 :termination ':primitive)))

(defun astar (i)
  (time
   (aha* (aref f i) :termination ':primitive)))

(defun ahss-first (i)
  (time
   (ahss (aref q i) '-infty :priority-fn #'priority-fn 
	 :termination ':begins-with-primitive)))

(defun run-offline (i)
  (format t "~&------------------------------~&AHSS-first~&")
  (ahss-first i)
  (format t "~&------------------------------~&AHSS~&")
  (ahs i)
  (format t "~&------------------------------~&AHA*~&")
  (aha i)
  (format t "~&------------------------------~&Flat A*~&")
  (astar i))
