(in-package geometry)



(defvar *canvas*)
(defvar *items*)
(defvar *zoom* 30)
(defvar *origin-x* 25)
(defvar *origin-y* 600)


(defun reset-ltk ()
  "restart the interface to LTK"
  ;; TODO make cleaner, remove hardcoded constants
  (exit-wish)
  (start-wish)
  (setf *items* nil)
  (let* ((sc (make-instance 'scrolled-canvas))
	 (c (canvas sc)))
    (pack sc :side :top :fill :both :expand 1)
    (setf *canvas* c)
    (ltk:scrollregion c 0 0 800 800)))



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Drawing particular shapes
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defun draw (poly &key (color "black") tags (width 2) (fill nil))
  (let ((items (designated-list (draw-shape poly))))
    (dolist (item items)
      (push item *items*)
      (itemconfigure *canvas* item "fill" (if fill color ""))
      (itemconfigure *canvas* item "outline" color)
      (when tags
	(itemconfigure *canvas* item "tags" tags))
      (when width
	(itemconfigure *canvas* item "width" width)))))

(defgeneric draw-shape (s)
  (:method (s) (declare (ignore s)))
  (:method :around (s) (declare (ignore s))
	   (when *wish* (call-next-method)))
  (:method ((s <polygon>))
    (create-polygon *canvas* (mapcans #'screen-coords (vertices s))))
  (:method ((s vector))
    (let* ((coords (screen-coords s))
	   (c1 (map 'vector #'1- coords))
	   (c2 (map 'vector #'1+ coords)))
      (apply #'create-oval *canvas* (concatenate 'list c1 c2))))
  (:method ((s <line-segment>))
    (create-line *canvas* (concatenate 'list (screen-coords (a s)) (screen-coords (b s)))))
  (:method ((s <axis-aligned-box>))
    (draw-shape (box->polygon s)))
  (:method ((s <implicit-union>))
    (let ((l nil))
      (do-elements (s2 (union-sets s) l)
	(push (draw-shape s2) l))))
  (:method ((s rigid-2d))
    (draw (centered-v s) :color "red" :fill t)
    (draw (make-instance '<line-segment> :a (centered-v s)
			 :b (a+ (centered-v s) (2d-rotate (rigid-2d-theta s) #(.5 0))))
	  :fill t)))
    


(defun clear-canvas ()
  "clear-canvas.  Remove all items from the LTK canvas."
  (while *items*
    (itemdelete *canvas* (pop *items*))))


(defun screen-coords (v)
  (dbind (x y) v
    (list (round (+ *origin-x* (* *zoom* x))) (round (- *origin-y* (* *zoom* y))))))


  
	   