(defpackage test-learn-hierarchy
  (:use 
   cl
   set
   rl-user
   utils
   td-taxi-env
   maxq)
  (:import-from 
   maxq
   <var-val-goal-language>
   empty-goal-description
   local-changes
   initial-hierarchy
   isomorphic
   make-var-var-goal
   make-var-val-set-goal
   make-var-specs
   add-task
   <task>
   
   parse-cost
   make-parse-tree
   is-valid-parse
   
   determinism-level
   determining-vars
   cond-indep-relevant-vars
   l2-histogram-feature-select
   l1-binned-feature-select
   
   ))

(in-package test-learn-hierarchy)

(defvar *num-episodes* 40)

(format t "~&Note: the tests are randomized since they depend on sampled runs of the policies.  So there may be occasional errors due to chance.")

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; isomorphism checking
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(setf hier1 (make-maxq ((top '(bar baz) nil)
			(bar '(baz) '(c d)))
		       '(baz qux))
      hier2 (make-maxq ((top '(bar baz) nil)
			(bar '(baz) '(c d)))
		       '(baz oof))
      hier3 (make-maxq ((top '(x baz) nil)
			(x '(baz) '(d c)))
		       '(qux baz))
      hier4 (make-maxq ((top '(foo bar) nil)
			(bar '(baz qux oof) '(y z))
			(baz '(gar oof) 3)
			(foo nil nil)
			(gar '(qux) 2))
		       '(oof qux))
      hier5 (make-maxq ((top '(afoo abar) nil)
			(agar '(qux) 2)
			(afoo nil nil)
			(abaz '(agar oof) 3)
			(abar '(abaz oof qux) '(y z)))
		       '(oof qux))
      hier6 (make-maxq ((top '(foo bar) nil)
			(foo nil nil)
			(bar '(baz qux oof) '(y z))
			(baz '(gar qux) 3)
			(gar '(oof) 2))
		       '(oof qux))
      hier7 (make-maxq ((top '(afoo abar) nil)
			(foo '(qux) 2)
			(afoo nil nil)
			(abaz '(agar oof) 3)
			(abar '(abaz oof qux) '(y z)))
		       '(oof qux))
      hier8 (make-maxq ((top '(bar baz oof) nil)
			(bar '(baz oof) '(c d))
			(oof '(qux) '(d e)))
		       '(baz qux))
		       
      )


(defun not-isomorphic (x y)
  (not (isomorphic x y)))

(tests 
 "Isomorphism checking"
 (hier1 hier2 #'not-isomorphic)
 (hier1 hier3 #'isomorphic)
 (hier2 hier3 #'not-isomorphic)
 (hier4 hier5 #'isomorphic)
 (hier4 hier6 #'not-isomorphic)
 (hier4 hier7 #'not-isomorphic))

(tests
 "Hierarchy operations"
 ((add-task (make-instance '<task> :children '(qux) :goal '(d e)) '(top bar) hier1) hier8 #'isomorphic)
 (hier1 hier3 #'not-isomorphic)
 (hier1 hier8 #'isomorphic))



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; goal language
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


(defun singletons (s)
  (mapset 'list #'list s))

(setf l (make-instance '<var-val-goal-language>
	  :var-specs (make-var-specs 
		      (foo #'first (singletons 3) 3)
		      (bar #'second (singletons '(a b)) 2)
		      (baz #'third (singletons nil) 1)))
      
      changes (local-changes l nil)
      s '(2 b b))


(do-tests "Simple goal language"
  (empty-goal-description l) nil
  (size changes) 10
  (sum-over changes #'(lambda (g) (indicator (member? s g))))
  4)


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; parsing
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(setf h1 (make-maxq ((top '(foo bar) nil)
		     (foo '(bar a) '(x y) '(z))
		     (bar '(baz qux) '(x z) '(y w))
		     (baz '(a b) '(w))
		     (qux '(c d) '(z u)))
		    '(a b c d))
      ngt 1
      gnt 10
      ii 100
      ic 1000
      traj1 (env-user:make-trajectory nil '(z) nil nil)
      actions1 nil
      tree1 (make-parse-tree traj1 '(top 0 0 ()))
      invalid1 (make-parse-tree traj1 '(foo 0 0 ()))
      traj2 (env-user:make-trajectory nil '(z w w z w u v u y) (make-list 8 :initial-element 'dummy)
				      (make-list 8 :initial-element -42))
      tree2 (make-parse-tree
	     traj2
	     '(top 0 8 ((foo 0 8 ((a 0 1 ())
				  (bar 1 3 ((baz 1 3 ((a 1 2 ())
						      (b 2 3 ())))))
				  (a 3 4 ())
				  (bar 4 8 ((qux 4 5 ((c 4 5 ())))
					    (baz 5 8 ((b 5 6 ())
						      (a 6 7 ())
						      (a 7 8 ()))))))))))
      tree3 (make-parse-tree
	     traj2
	     '(top 0 8 ((foo 0 8 ((a 0 1 ())
				  (bar 1 3 nil)
				  (a 3 4 ())
				  (bar 4 8 ((qux 4 5 ((c 4 5 ())))
					    (baz 5 8 ((b 5 6 ())
						      (a 6 7 ())
						      (a 7 8 ()))))))))))
      
      invalid2 (make-parse-tree
		traj2
	     '(top 0 8 ((foo 0 8 ((a 0 1 ())
				  (bar 1 3 ((baz 1 3 ((a 1 2 ())
						      (b 2 3 ())))))
				  (a 3 5 ())
				  (bar 4 8 ()))))))
      
      invalid3 (make-parse-tree
		traj2
	     '(top 0 7 ((foo 0 8 ((a 0 1 ())
				  (bar 1 3 ((baz 1 3 ((a 1 2 ())
						      (e 2 3 ())))))
				  (a 3 4 ())
				  (bar 4 8 ()))))))
      
      invalid4 (make-parse-tree
		traj2
	     '(top 0 7 ((foo 0 8 ((a 0 1 ())
				  (bar 1 3 ((baz 1 3 ((a 1 2 ())
						      (b 2 3 ())))))
				  (a 3 4 ())
				  (b 4 8 ()))))))
      
      tree4 (make-parse-tree
	     traj2
	     '(top 0 8 ((foo 0 8 ((a 0 1 ())
				  (baz 1 3 ((a 1 2 ())
					    (b 2 3 ())))
				  (a 3 4 ())
				  (bar 4 8 ((qux 4 5 ((c 4 5 ())))
					    (baz 5 8 ((b 5 6 ())
						      (a 6 7 ())
						      (a 7 8 ()))))))))))
      
      )
					   
				      

(defun valid-parse (p)
  (is-valid-parse p h1))

(defun invalid-parse (p)
  (not (valid-parse p)))

(tests
 "Validate parses"
 (tree1 #'valid-parse)
 (invalid1 #'invalid-parse)
 (tree2 #'valid-parse)
 (tree3 #'valid-parse)
 (invalid2 #'invalid-parse)
 (invalid3 #'invalid-parse)
 (invalid3 #'invalid-parse))


(defun pc (tree  h)
  (parse-cost tree h :non-goal-term-cost ngt :goal-non-term-cost gnt
	      :incorrect-child-cost ic :incorrect-initiation-cost ii))
	      

(tests 
 "Evaluate parses"
 ((pc tree1 h1) 0)
 ((pc tree2 h1) 110)
 ((pc tree3 h1) 0)
 ((pc tree4 h1) 1111)
 ((results-in-error error (pc invalid2 h1)) t))



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; finding relevant variables using determinism/
;; conditional independence
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(setf examples '(((foo 2 a) . 1) ((foo 5 b) . 27) ((bar 7 a) . 4) ((bar 1 c) . 9)
		 ((foo 5 a) . 1) ((foo 1 a) . 2) ((foo 7 a) . 1) ((bar 6 c) . 9)
		 ((bar 2 c) . 8) ((bar 1 c) . 9) ((bar 5 c) . 9))
      vs (make-var-specs (u #'first nil nil) (v #'second nil nil) (w #'third nil nil)))

(tests
 "Relevance determination"
 ((determinism-level examples (list #'third #'first) #'eql) 3/4)
 ((determinism-level examples (list #'first #'third #'second) #'equal) 1)
 ((let ((*determinism-threshold* .9))
    (determining-vars examples vs))
  '(u v w) #'set-eq)
 ((cond-indep-relevant-vars examples vs) '(u v w) #'set-eq))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; piecewise-constant l2 feature selection
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(setf examples '(((foo qux 4) . 9) 
		 ((foo qux 4) . 8)
		 ((bar oof 2) . 5)
		 ((bar ooof 2) . 4)
		 ((foo qux 6) . -10)
		 ((bar qux 4) . 6)))

(tests
 "Feature selection"
 ((l1-binned-feature-select examples vs :var-cost 0) '(u v w) #'set-eq)
 ((l1-binned-feature-select examples vs :var-cost .05) '(u w) #'set-eq)
 ((l1-binned-feature-select examples vs :var-cost 1) '(w))
 ((l1-binned-feature-select examples vs :var-cost 20) nil))
