(defpackage test-maxq
  (:use 
   cl
   set
   rl-user
   td-taxi-env
   prob
   utils
   maxq)
  (:import-from    
   maxq
   <var-val-goal-language>
   make-var-val-set-goal
   make-var-var-goal
   make-var-specs
   
   initial-hierarchy
   greedy-hierarchy-learning
   
   *var-cost*
   *goal-language*
   *primitive-actions*
   *trajectories*
   *var-specs*
   *h*
   
   ))

(in-package test-maxq)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; params
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(declare-params 
 (*num-episodes* 200)
 (*num-domains* 3)
 (*map-size* 4)
 (*learning-length* 75000)
 (*num-sources* 3)
 (*num-dests* 3)
 (*msp* .9))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; taxi example env set up
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defun singletons (s) (make-instance '<subsets-of-size> :base-set s :size 1))

(setf locs (direct-product 'list *map-size* *map-size*)
      source-sampler (make-instance '<sampling-without-replacement> 
		       :dist (make-unif-dist-over-set locs)
		       :n *num-sources* :element-type 'list)
      dest-sampler (make-instance '<sampling-without-replacement> 
		       :dist (make-unif-dist-over-set locs)
		       :n *num-dests* :element-type 'list)
      
      sources (mapset 'list #'(lambda (i) (sample source-sampler)) *num-domains*)
      dests (mapset 'list #'(lambda (i) (sample dest-sampler)) *num-domains*)
      all-sources (remove-duplicates (apply #'append sources) :test #'equal)
      all-dests (remove-duplicates (apply #'append dests) :test #'equal)
      
      envs (mapcar
	    #'(lambda (s d)
		(make-instance '<td-taxi-env> :world-map (make-array (list *map-size* *map-size*) :initial-element t)
			       :ps s :pd d :msp *msp*))
	    sources dests)
      ce (make-instance 'certainty-equivalence:<certainty-equivalence> :unknown-state-behavior ':error)
      vs (make-var-specs
	  (taxi-pos #'taxi-pos (singletons locs) (size locs))
	  (pass-loc #'pass-loc (singletons '(in-taxi at-source at-dest)) 3)
	  (pass-source #'pass-source (singletons all-sources) *num-sources*)
	  (pass-dest #'pass-dest (singletons all-dests) *num-dests*))
      
      tl (make-instance '<var-val-goal-language>
	   :var-specs vs)
      prim '(n w s e p d)
      )


(format t "generating trajectories")
(setf tr nil)
(dotimes (i *num-domains*)
  (format t "~&domain ~a. learning" i)
  (rl-user:learn (elt envs i) 'random ce *learning-length* :hist-length 1 :step-print-inc 2000 :ep-print-inc nil)
  (format t " gathering trajectories")
  (setf e (elt envs i)
	pol (make-instance 'exp-pol:<boltzmann-exp-pol> :q-learning-alg ce :temp-fn (constantly .01))
	q (current-q-fn ce))
  (setf tr (concatenate 'vector tr (rl-user:gather-trajectories (elt envs i) pol
								:num-episodes *num-episodes*))))



(setf maxq::*var-cost* .1)


(setf trans (ndunion ((traj tr)) (env:transitions traj :share-structure t))
      atr (mapcar #'(lambda (act)
		      (cons act
			    (ndlet-fail ((tr trans))
			      (dbind (s a r s2) tr
				(if (eq a act)
				    (cons s s2)
				  'fail)))))
		  '(n s e w p d))
      vars `((pos . ,#'taxi-pos)
	     (loc . ,#'pass-loc)
	     (src . ,#'pass-source)
	     (dst . ,#'pass-dest)))