(defpackage test-rbe-hlearn
  (:nicknames trh)
  (:use cl
	utils
	q-learning
	set
	rbe
	maxq
	alisp-prog
	alisp-features
	alisp-user))

(in-package trh)


(declare-params
 (*num-steps* 50000)
 (*num-trials* 5)
 (*hist-length* 50)
 (*goal-res* 2))

(setf wm (make-array '(3 3) :initial-element 't))
(setf num-peas 1)
(setf base-loc '(0 0))
(setf forest-locs '((0 2)))
(setf mine-locs '((2 0) (2 2)))

(setf e (make-rbe-env wm num-peas *goal-res* *goal-res* base-loc forest-locs mine-locs :slip-prob .01 :dropoff-reward 0))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; env accessors 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defun my-pos ()
  (aref (pos (env-state)) 0))

(defun my-status ()
  (aref (status (env-state)) 0))

(defun at-gold-loc ()
  (member (my-pos) mine-locs :test #'equal))

(defun at-wood-loc ()
  (member (my-pos) forest-locs :test #'equal))

(defun at-base-loc ()
  (equal (my-pos) base-loc))

(defun dropped-off-gold ()
  (eq (my-status) 'dropoff-gold))

(defun dropped-off-wood ()
  (eq (my-status) 'dropoff-wood))

(defun have-gold ()
  (eq (my-status) 'carrying-gold))

(defun have-wood ()
  (eq (my-status) 'carrying-wood))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; partial program
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defun top ()
  (loop
    (choose 
     task-choice
     (call (gather-gold))
     (call (gather-wood)))))

(defun gather-gold ()
  (repeat-until (dropped-off-gold)
    (choose
     gather-gold-choice
     (call (get-gold))
     (call (put-gold)))))

(defun gather-wood ()
  (repeat-until (dropped-off-wood)
    (choose
     gather-wood-choice
     (call (get-wood))
     (call (put-wood)))))

(defun get-gold ()
  (repeat-until (have-gold)
    (choose
     gold-choice
     (nav (nav-gold-loc))
     (pickup (action (list (cons 0 (if (at-gold-loc) 'P 'R))))))))

(defun get-wood ()
  (repeat-until (have-wood)
    (choose
     wood-choice
     (nav (nav-wood-loc))
     (pickup (action (list (cons 0 (if (at-wood-loc) 'P 'R))))))))

(defun nav-gold-loc ()
  (repeat-until (at-gold-loc)
    (action
     `((0 . ,(choose
	      nav-gold-choice
	      (N 'N) (S 'S) (E 'E) (W 'W)))))))

(defun nav-wood-loc ()
  (repeat-until (at-wood-loc)
    (action
     `((0 . ,(choose
	      nav-wood-choice
	      (N 'N) (S 'S) (E 'E) (W 'W)))))))

(defun nav-base-loc ()
  (repeat-until (at-base-loc)
    (action
     `((0 . ,(choose
	      nav-base-choice
	      (N 'N) (S 'S) (E 'E) (W 'W)))))))


(defun put-gold ()
  (repeat-until (not (have-gold))
    (choose
     put-gold-choice
     (nav (nav-base-loc))
     (dropoff (action '((0 . D)))))))

(defun put-wood ()
  (repeat-until (not (have-wood))
    (choose
     put-wood-choice
     (nav (nav-base-loc))
     (dropoff (action '((0 . D)))))))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; features
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defvar *featurizer*)

(setq *featurizer*
  (make-3partq-featurizer
   ()
   (task-choice
    (:qr-depends peasant-loc choice)
    (:qc-depends gold-amount wood-amount choice))
   (gather-gold-choice
    (:qr-depends peasant-status peasant-loc choice)
    (:qc-depends peasant-status choice))
   (gather-wood-choice
    (:qr-depends peasant-status peasant-loc choice)
    (:qc-depends peasant-status choice))
   (gold-choice
    (:qr-depends peasant-status peasant-loc choice)
    (:qc-depends choice peasant-loc peasant-status))
   (wood-choice
    (:qr-depends peasant-status peasant-loc choice)
    (:qc-depends peasant-status choice peasant-loc))
   (nav-gold-choice
    (:qr-depends)
    (:qc-depends peasant-loc choice))
   (nav-wood-choice
    (:qr-depends)
    (:qc-depends peasant-loc choice))
   (nav-base-choice
    (:qr-depends)
    (:qc-depends peasant-loc choice))
   (put-gold-choice
    (:qr-depends peasant-status peasant-loc choice)
    (:qc-depends peasant-status peasant-loc choice))
   (put-wood-choice
    (:qr-depends peasant-status peasant-loc choice)
    (:qc-depends peasant-status peasant-loc choice))))

   


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Definitions of the individual features
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(def-feature peasant-loc (omega u)
  (aref (pos env-state) 0))

(def-feature gold-amount (omega u)
  (gold env-state))

(def-feature wood-amount (omega u)
  (wood env-state))

(def-feature peasant-status (omega u)
  (aref (status env-state) 0))



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; algs
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(setf p #'top)
(setf hq (make-instance 'ahq:<hordq> :lrate .4))
(setf hqa (make-instance 'ahq:<hordq> :features *featurizer* :lrate .4))
(setf q-learner (make-instance '<q-learning> :env e :lrate .4 :discount 1 :unknown-state-behavior ':random))


(defun foo ()
  (alisp-user:learn p e (make-instance 'exp-pol:<boltzmann-exp-pol>
			  :q-learning-alg hqa
			  :temp-fn (constantly 0))
			  (list hqa) *num-steps* :hist-length *hist-length*)
  (setf hqa-rews (alisp-user:evaluate p e (alisp-user:get-policy-hist hqa) :num-steps 100 :num-trials *num-trials*)))

(defun bar ()
  (rl-user:learn e (make-instance 'exp-pol:<boltzmann-exp-pol>
		     :q-learning-alg q-learner
		     :temp-fn (constantly 0))
		 (list q-learner) *num-steps* :hist-length *hist-length*)
  (setf q-rews (rl-user:evaluate e (rl-user:get-policy-hist q-learner) :num-steps 100 :num-trials *num-trials*)))


