(defpackage test-strat
  (:use cl
	utils
	s-env
	create-env
	prob
	alisp-prog)
  (:import-from alisp-user
		*pause-after-actions*)
  (:shadowing-import-from alisp-user
			  io-interface
			  gather-trajectories))

(in-package test-strat)

(defclass <build-barracks-env> (<stratagus-env>) ())

(defmethod is-terminal-state ((e <build-barracks-env>) s)
  (units-belonging-to-of-type (pid e) 's-env::human-barracks s))
  

(defparameter *gold1* '(16 20))
(defparameter *gold2* '(24 16))
(defparameter *wood1* '(18 5))
(defparameter *wood2* '(9 19))
(defparameter *base* '(2 11))
(defparameter *barracks-site* '(5 3))

(setf e (make-instance '<build-barracks-env>))

(defun build-barracks ()
  (loop
    (let ((s (env-state)))
      (when (and (>= (gold s) 700) (>= (wood s) 450))
	(return))
      (ecase (random 2)
	(0 (when (< (gold s) 700) (gather-gold)))
	(1 (when (< (wood s) 450) (gather-wood))))))
  (navigate *barracks-site*)
  (peasant-action 'build 'barracks)
  (wait-till-free))

(defun gather-gold ()
  (navigate (sample-uniformly (list *gold1* *gold2*)))
  (peasant-action 'harvest 'gold)
  (wait-till-free)
  (navigate *base*)
  (peasant-action 'dropoff)
  (wait-till-free))

(defun gather-wood ()
  (navigate (sample-uniformly (list *wood1* *wood2*)))
  (peasant-action 'harvest 'wood)
  (wait-till-free)
  (navigate *base*)
  (peasant-action 'dropoff)
  (wait-till-free))


(defun wait-till-free ()
  (until (eq (get-unit-status 1 (env-state)) 'stop)
    (action 'noop)))

(defun peasant-action (&rest args)
  (action (list (cons 1 args))))

(defun navigate (dest)
  (until (equal (get-unit-loc 1 (env-state)) dest)
    (peasant-action 'move-towards dest)))