(defpackage test-task-learner
  (:use 
   cl
   set
   env-user
   td-taxi-env
   prob
   utils
   maxq)
  (:import-from    
   maxq
   <var-val-goal-language>
   make-var-val-set-goal
   make-var-var-goal
   make-var-specs
   
   initial-hierarchy
   greedy-hierarchy-learning
   
   
   <task>
   <subtask-env>
   task-learner
   ))

(in-package test-task-learner)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; params
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(declare-params 
 (*map-size* 4)
 (*num-steps-learning* 20000)
 (*num-sources* 3)
 (*num-dests* 3)
 (*msp* .9)
 (*lrate* .1)
 (*goal-val* 2))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; taxi example env set up
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defun singletons (s) (make-instance '<subsets-of-size> :base-set s :size 1))

(setf locs (direct-product 'list *map-size* *map-size*)
      source-sampler (make-instance '<sampling-without-replacement> 
		       :dist (make-unif-dist-over-set locs)
		       :n *num-sources* :element-type 'list)
      dest-sampler (make-instance '<sampling-without-replacement> 
		     :dist (make-unif-dist-over-set locs)
		     :n *num-dests* :element-type 'list)
      
      sources (sample source-sampler)
      dests (sample dest-sampler)

      env (make-instance '<td-taxi-env> :world-map (make-array (list *map-size* *map-size*) :initial-element t)
			 :ps sources :pd dests :msp *msp*)
      ce (make-instance 'certainty-equivalence:<certainty-equivalence> :unknown-state-behavior ':error)
      vs (make-var-specs
	  (taxi-pos #'taxi-pos (singletons locs) (size locs))
	  (pass-loc #'pass-loc (singletons '(in-taxi at-source at-dest)) 3)
	  (pass-source #'pass-source (singletons sources) *num-sources*)
	  (pass-dest #'pass-dest (singletons dests) *num-dests*))
      
      tl (make-instance '<var-val-goal-language>
	   :var-specs vs)

      pass-in-taxi (make-var-val-set-goal pass-loc '(in-taxi))
      taxi-at-dest (make-var-var-goal taxi-pos pass-dest)
      taxi-at-dest-with-pass (filter ':implicit pass-in-taxi #'(lambda (x) (member? x taxi-at-dest)))
      
      child-task (make-instance '<task> :children '(n s w e p) :goal pass-in-taxi)
      task (make-instance '<task> :children '(get-pass n s w e) :goal taxi-at-dest-with-pass :init-set (complement-set taxi-at-dest))
      
      pol #'(lambda (s)
	      (dbind (tr tc) (taxi-pos s)
		(dbind (pr pc) (pass-source s)
		  (cond
		   ((< tr pr) 'S)
		   ((> tr pr) 'N)
		   ((< tc pc) 'E)
		   ((> tc pc) 'W)
		   (t 'P)))))
      
      s1 (get-state env)
      s2 (progn (env:reset env) (get-state env))
      subtasks `((get-pass ,(complement-set pass-in-taxi) ,pol))
      
      e (make-instance '<subtask-env> :parent-env env :init-states (list s1 s2)
		       :subtasks subtasks
		       :prim '(n w s e) :app-set (complement-set taxi-at-dest)))


(defun foo ()
  (task-learner env (list s1 s2) (complement-set taxi-at-dest) taxi-at-dest-with-pass *goal-val* '(n w s e) subtasks *num-steps-learning* :learning-rate *lrate*))
		      
		