(in-package maxq)

(defstruct (var-spec (:conc-name vs-) (:constructor make-var-spec (function allowed-goals &key expected-size)))
  function
  allowed-goals
  expected-size)



(defmacro make-var-specs (&rest args)
  "make-var-specs (VAR1 FN1 GOALS1 SIZE1) ... (VARk FNk GOALSk SIZEk)
VARi are not evaluated, and must be either symbols or lists of objects that are either symbols or numbers.  The FNi and GOALSi are evaluated."
  
  (assert
      (every 
       #'(lambda (a) 
	   (let ((var (first a))) 
	     (or (symbolp var) 
		 (and (listp var) 
		      (every #'(lambda (x) (or (symbolp x) (numberp x))) var)))))
       args)
      nil "Var names ~a must be symbols or lists of symbols or numbers." (mapcar #'car args))
  `(list
    ,@(mapcar
       #'(lambda (a)
	   (dbind (name fn goals expected-size) a
	     `(cons ',name (make-var-spec ,fn ,goals :expected-size ,expected-size))))
       args)))



(defun var-function (v var-specs)
  (vs-function (mapping:evaluate var-specs v)))

(defun var-allowed-goals (v var-specs)
  (vs-allowed-goals (mapping:evaluate var-specs v)))

(defun var-expected-size (v var-specs)
  (vs-expected-size (mapping:evaluate var-specs v)))

(defun all-vars (var-specs)
  (mapping:domain var-specs))