(in-package mdp)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Types, basic accessors
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defclass <sparse-mdp> (<mdp>)
  ((state-descs :accessor state-descs :initarg :state-descs)
   (init-dist :initform 'undefined)
   (state-test :initarg :state-test :initform #'equalp)
   (action-test :initarg :action-test :initform #'equal))
  (:documentation "<sparse-mdp> (<mdp>

Initargs
:state-descs - a hash table, where the keys are states, and the values are lists consisting of 1) a boolean denoting whether the state is terminal 2) a hash table, where the keys are action names and the values are lists consisting of a [prob-dist] objects over next state, and an association list from next state to reward 

For now, all equality tests between states use #'equalp and equality tests between states use #'equal."))


(declaim (inline state-desc state-action-desc))

(defun state-desc (m s)
  (gethash s (state-descs m)))

(defun state-action-desc (m s a)
  (gethash a (second (state-desc m s))))

(defmethod state-set ((m <sparse-mdp>))
  (hash-keys (state-descs m)))

(defmethod avail-actions ((m <sparse-mdp>) s)
  (hash-keys (second (state-desc m s))))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; conversion
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defun sparsify-dist (d)
  (let ((l nil))
    (map-prob-dist #'(lambda (x p) (when (> p 0) (push (cons x p) l))) d)
    l))

(defun sparsify-mdp (m)
  (let ((h (make-hash-table :test #'equalp)))
    (do-elements (s (state-set m) (make-instance '<sparse-mdp> :state-descs h))
      (let ((h2 (make-hash-table :test #'equal)))
	(do-elements (a (avail-actions m s))
	  (let ((d (sparsify-dist (trans-dist m s a))))
	    (setf (gethash a h2)
	      (list d (mapset 'list #'(lambda (s2) (cons s2 (reward m s a s2)))
			      (sample-space d))))))
	(setf (gethash s h) (list (terminal? m s) h2))))))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; mdp ops
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defmethod terminal? ((m <sparse-mdp>) s)
  (first (state-desc m s)))

(defmethod trans-dist ((m <sparse-mdp>) s a)
  (first (state-action-desc m s a)))

(defmethod reward ((m <sparse-mdp>) s a d)
  (mapping:evaluate (second (state-action-desc m s a)) d))  

