;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Various operations that wrap the basic dmcmc filter
;; to make experiments easier to run
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


(in-package dmcmc)

(defun single-timestep-estimates (ssm decay-rate obs fn ns)
  "single-timestep-estimates SSM DECAY-RATE OBS FN NS

The first three arguments are as in the initargs to <dmcmc>.  FN is a function that takes in a probability distribution and returns any object.  NS is a list of positive integers in ascending order.  

Runs dmcmc for (last ns) samples, and returns vector of the results of applying FN to the Monte Carlo estimates at intermediate points corresponding to the sample numbers in NS."
  
  (let ((hist-gatherer (make-instance '<estimate-history> :estimate-timesteps ns :fn fn)))
    (generate-samples 
     (make-instance '<dmcmc> :ssm ssm :decay-rate decay-rate :obs obs :observers (list hist-gatherer))
     (slast ns))
    (estimate-history hist-gatherer)))


(defun convergence-curve (ssm decay-rate obs ns &key (true-dist nil))
  "convergence-curve SSM DECAY-RATE OBS NS &key TRUE-DIST
First three args are as in initargs to <dmcmc> and NS is an ascending list of positive integers.
Runs DMCMC and returns vector representing the total variation distance from the true answer as a function of number of samples.

If TRUE-DIST is not provided, it will be computed using exact filtering."

  (orf true-dist (exact-filter:exact-filter  ssm obs (1- (length obs))))
  (single-timestep-estimates ssm decay-rate obs #'(lambda (d) (total-variation-distance d true-dist)) ns))



