(defpackage test-dmcmc
  (:use
   prob
   cl
   utils
   set
   ssm
   mapping
   exact-filter
   pf
   dmcmc))


(in-package test-dmcmc)

(declare-params
 (*num-samples* 100000 "Number of samples per dmcmc trial")
 (*num-particles* 100000 "Number of particles used in pf")
 (*num-trials* 10)
 (*num-points-on-learning-curve* 20)
 (*trans-det* .4)
 (*obs-det* .9)
 (*run* nil)
 (*pf-trans* ':posterior))




(setf init-dist (make-unif-dist-over-set '((0 0) (0 1) (1 0) (1 1)))
      trans-dist #'(lambda (u)
		     (let ((v (copy-list u))
			   (w (copy-list u)))
		       (setf (first v) (- 1 (first v))
			     (second w) (- 1 (second w)))
		       (p2alist u *trans-det* v (/ (- 1 *trans-det*) 2) w (/ (- 1 *trans-det*) 2))))
      obs-dist #'(lambda (u)
		   (dbind (x y) u
		     (let ((ind (indicator (= x y))))
		       (p2alist ind *obs-det* (- 1 ind) (- 1 *obs-det*)))))
      ssm (make-instance '<state-space-model> :init-dist init-dist :trans-dist trans-dist :obs-dist obs-dist)
      obs #(1 1 1 0 1 1 0 0 0 1)
      ef (make-instance '<exact-filter> :ssm ssm)
      ns (round *num-samples* *num-points-on-learning-curve*)
      rv2 #'(lambda (x) (apply #'* x))
      rv #'(lambda (x) (evaluate-rv rv2 (elt x 9)))
      df (make-instance 'dmcmc:<dmcmc-filter> :decay-rate (polynomial-decay 2 10) :num-samples *num-samples* :ssm ssm)
      pf (make-instance 'particle-filter:<particle-filter> :num-particles *num-particles* :ssm ssm 
			:resampling-strategy ':correlated :trans-strategy *pf-trans*))




