(defpackage exact-filter
  (:documentation "Package exact-inference

Types
-----
<exact-filter>
<kalman-filter>

Operations
----------
exact-filter
")
  (:export
   
   exact-filter
   <exact-filter>
   <kalman-filter>
   )
  (:use
   cl
   utils
   prob
   ssm
   mapping
   set))

(in-package exact-filter)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Batch filtering
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defun exact-filter (ssm observations ts)
  "exact-filter SSM OBSERVATIONS TIMESTEP
Return the posterior distribution of the hidden state at the given timestep, conditional on the evidence up to that timestep."
  (let ((dist (init-dist ssm))
	(trans (trans-dist ssm))
	(obs (obs-dist ssm)))

    (_f condition-on-dist dist obs (evaluate observations 0))
    (dotimes (i ts dist)
      (_f chain-dists dist trans)
      (_f marginalize dist #'second)
      (_f condition-on-dist dist obs (evaluate observations (1+ i))))))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Online filtering
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defclass <exact-filter> ()
  ((ssm :accessor ssm :initarg :ssm)
   (dist :accessor current-state-dist)
   (first-step? :accessor first-step?)))

(defmethod initialize-instance :after ((f <exact-filter>) &rest args)
  (declare (ignore args))
  (reset-filter f))

(defmethod reset-filter ((f <exact-filter>))
  (setf (first-step? f) t
	(current-state-dist f) (init-dist (ssm f))))

(defmethod incorporate-evidence ((f <exact-filter>) o)
  (if (first-step? f)
      (setf (first-step? f) nil)
    (progn
      (_f transform-dist (current-state-dist f) (trans-dist (ssm f)))
      (debug-print 4 "Stepped distribution forward to ~/prob:pprint-prob-dist/" (current-state-dist f))))
  
  (_f condition-on-dist (current-state-dist f) (obs-dist (ssm f)) o)
  (debug-print 4 "Conditioned on ~a to get dist ~/prob:pprint-prob-dist/" o (current-state-dist f)))
  
  
  
  
  

