(defpackage dmcmc-exp1
  (:use
   prob
   cl
   utils
   set
   ssm
   mapping
   dmcmc))


(in-package dmcmc-exp1)

(declare-params
 (*num-samples* 100000 "Number of samples per trial")
 (*num-trials* 10)
 (*num-points-on-learning-curve* 20)
 (*trans-det* .4)
 (*obs-det* .9))


(setf init-dist (make-unif-dist-over-set '((0 0) (0 1) (1 0) (1 1)))
      trans-dist #'(lambda (u)
		     (let ((v (copy-list u))
			   (w (copy-list u)))
		       (setf (first v) (- 1 (first v))
			     (second w) (- 1 (second w)))
		       (p2alist u *trans-det* v (/ (- 1 *trans-det*) 2) w (/ (- 1 *trans-det*) 2))))
      obs-dist #'(lambda (u)
		   (dbind (x y) u
		     (let ((ind (indicator (= x y))))
		       (p2alist ind *obs-det* (- 1 ind) (- 1 *obs-det*)))))
      ssm (make-instance '<state-space-model> :init-dist init-dist :trans-dist trans-dist :obs-dist obs-dist)
      obs #(1 1 1 0 1 1 0 0 0 1)
      exact (exact-inference:exact-filter ssm obs 9)
      ns (round *num-samples* *num-points-on-learning-curve*)
      rv2 #'(lambda (x) (apply #'* x))
      rv #'(lambda (x) (evaluate-rv rv2 (elt x 9)))
      mc (make-instance '<monte-carlo-observer> :rv rv)
      d (make-instance '<dmcmc> :ssm ssm :obs obs :observers (list mc)))




(format t "~&Exact expectation is ~a" (expectation exact #'(lambda (x) (apply #'* x))))

(mvbind (v avg std)
    (randomized-trials
     *num-trials*
     (reset-sample d)
     (prog1
	 (mapset 
	  'vector
	  #'(lambda (i)
	      (declare (ignore i))
	      (dotimes (j ns)
		(draw-sample d))
	      (mc-estimate mc))
	  *num-points-on-learning-curve*)
       (reset-observer mc)))
  
  (format t "~&MCMC estimates taken at intervals of ~a steps and averaged over ~a trials.~&Estimate averages are ~a~&Estimate standard deviations are ~a" ns *num-trials* (round-array avg) (round-array std)))


(mvbind (v avg std)
    (randomized-trials 
     *num-trials* 
     (setf pf (make-instance 'particle-filter:<particle-filter> :num-particles np :ssm ssm)) 
     (particle-filter::reset-samples pf) 
     (float (/ (sum-over (particle-filter::sir pf obs) #'(lambda (x) (evaluate-rv rv2 x))) np)))

  (format t "~&PF estimates have avg ~a and std ~a" avg std))
      
      
			 
			 
