(in-package ssm)

(defgeneric reset-filter (f)
  (:documentation "reset-filter FILTER.  All filters must implement this operation, which tells the filter that a new trajectory has just begun."))

(defgeneric incorporate-evidence (f obs)
  (:documentation "incorporate-evidence FILTER OBSERVATION.  Inform filter of next observation."))

(defgeneric current-state-dist (f)
  (:documentation "current-state-dist FILTER.  Return [prob-dist] over the state at the current time, i.e., the time at which the last observation was reached.  If no observations have happened yet, return the distribution over the state at time 0."))

(defun filter-observations (f obs &optional (reset-first t))
  "filter-observations FILTER OBSERVATIONS &optional (RESET-FIRST t).

Returns the distribution after filtering the given OBSERVATIONS, starting at either the initial distribution (if RESET-FIRST) or the current state distribution."
  (when reset-first (reset-filter f))
  (dotimes (i (length obs))
    (debug-print 1 "Dist is~&~2I~/prob:pprint-prob-dist/"  (current-state-dist f))
    (debug-print 0 "Incorporating observation ~a" i)
    (incorporate-evidence f (aref obs i)))
  (debug-print 1 "Final dist is ~/prob:pprint-prob-dist/" (current-state-dist f)))

(defgeneric incorporate-evidence-sequence (f obs)
  (:documentation "incorporate-evidence-sequence F OBS.  Incorporate the sequence of observations in OBS.  By default, just incorporates them one at a time, but some filters may override this.  Return the new distribution.")
  (:method (f obs)
	   (filter-observations f obs nil)
	   (current-state-dist f)))