(defpackage test-pf
  (:use
   prob
   cl
   utils
   set
   ssm
   mapping
   exact-filter
   pf
   dmcmc))


(in-package test-pf)

(declare-params
 (*num-particles* 1000 "Number of particles used in pf")
 (*pf-trans* ':posterior)
 (*m* #(3 4))
 (*c* #2A((1 -2) (-2 20)))
 (*o* #2A((3 -2) (1 4) (5 -6)))
 (*ob* #(7 -2 -3))
 (*ov* #2A((4 -4 -2) (-4 13 14) (-2 14 18)))
 (*t* #2A((.6 .2) (-.2 .3)))
 (*tb* #(3 7))
 (*tv* #2A((0.04 0.2) (0.2 10))))

(setf lds (make-lds :init-mean *m* :init-cov *c* :trans *t* :trans-bias *tb* :trans-cov *tv* :obs *o* :obs-bias *ob* :obs-cov *ov*))

(defun kalman (obs)
  (filter-observations (make-instance '<exact-filter> :ssm lds) obs))

(defun pf (obs)
  (filter-observations (make-instance '<particle-filter> :ssm lds :num-particles *num-particles* :trans-strategy *pf-trans*) obs))

(mvsetq (st obs) (sample-finite-trajectory lds 20))

(format t "~&Generated state-observation sequence ~a" (map 'list #'cons st obs))
(format t "~&Kalman filtering dist is ~/prob:pprint-prob-dist/" (kalman obs))
(setf pd (pf obs)
      pm (make-array 2)
      pc (make-array '(2 2)))

(dotimes (i 2)
  (setf (aref pm i) (expectation pd #'(lambda (x) (aref x i)))))

(dotimes (i 2)
  (dotimes (j 2)
    (setf (aref pc i j) (- (expectation pd #'(lambda (x) (* (aref x i) (aref x j)))) (* (aref pm i) (aref pm j))))))


(format t "~&PF dist has mean ~a, and covariance ~a" pm pc)