(defpackage test-advantage-function
  (:use cl
	utils
	advantage-function)
  )

(in-package test-advantage-function)

(setf adv (make-instance '<tabular-advantage-function>
	    :avail-actions-specifier 
	    (constantly '(N S E W))))

(do-tests "tabular advantage functions"
  (array-dimensions (adv-fn::table adv)) '(0 0)
  
  (progn
    (update adv '(1 2) 'E 1 .1)
    (update adv '(1 2) 'W .5 .1)
    (handler-bind ((unknown-action #'(lambda (c) (use-value '-infty))))
      (max-advantage adv '(1 2))))
  1.0
  
  (max-advantage adv '(2 2))
  0
  
  (progn
    (update adv '(1 2) 'W -9.5 .2)
    (evaluate adv '(1 2) 'W))
  -1.5
  
  (progn
    (handler-bind ((unknown-action #'add-to-table)) (normalize-state adv '(1 2)))
    (list (evaluate adv '(1 2) 'E) (evaluate adv '(1 2) 'W)))
  '(0 -2.5)
  
  (progn (setf pol (make-instance '<advantage-maximizing-policy> :adv-fn adv))
	 (policy:make-choice pol '(1 2)))
  'E
  
  (progn (setf adv2 (clone adv))
	 (setf pol2 (make-instance '<advantage-maximizing-policy> :adv-fn adv2))
	 (update adv2 '(1 2) 'E -4 1)
	 (policy:make-choice pol2 '(1 2)))
  'W
  
  (policy:make-choice pol '(1 2))
  'E)
      


(setf adv3 (clone (sum-advantage-functions adv adv2)))


(do-tests "sum advantage functions"
  (evaluate adv3 '(1 2) 'E) -4
  (evaluate adv3 '(1 2) 'W) -5
  (max-advantage adv3 '(1 2)) -4
  
  (progn
    (normalize-state adv3 '(1 2))
    (evaluate adv3 '(1 2) 'E))
  0
  
  (evaluate adv3 '(1 2) 'W) -1
  
  (evaluate-comps adv3 '(1 2) 'E) '(2 -2)
  (evaluate-comps adv3 '(1 2) 'W) '(-.5 -.5))



