(defpackage difference-function
  (:documentation "
Package difference-function (diff-fn)

Types
-----
<tabular-difference-function>

Operations
----------
difference
update

Conditions
----------
unknown-difference
")
  (:export
   <difference-function>
   <tabular-difference-function>
   
   difference
   update
   unknown-difference)
  (:nicknames diff-fn)
  (:use
   cl
   set
   utils
   value-fn
   prob))


(in-package diff-fn)

(define-condition unknown-difference (error)
  ((diff-fn :initarg :diff-fn :reader diff-fn)
   (state1 :initarg :state1 :reader state1)
   (state2 :initarg :state2 :reader state2))
  (:report (lambda (c s) (format s "Difference function ~a unknown at ~a, ~a"
				 (diff-fn c) (state1 c) (state2 c)))))

(defgeneric difference (d s s2 &optional default-val)
  (:documentation "difference DIFF-FN S S2 &optional (DEFAULT-VAL nil)
Difference in value of S and S2.  DIFF-FN can be any difference function (possibly implicitly represented as a value function.)  If DEFAULT-VAL is not nil, then return it at unknown states.  If it is nil, may throw an unknown-difference condition.")
  (:method ((d <value-fn>) s s2 &optional (default-val nil))
	   (handler-case
	       (- (value d s2) (value d s))
	     (unknown-state ()
	       (aif default-val it (error 'unknown-difference :diff-fn d :state1 s :state2 s2))))))

(defgeneric update (d s s2 target step-size)
  (:documentation "update DIFF-FN S S2 TARGET STEP-SIZE.
DIFF-FN must be an actual difference function (not a value function).  STEP-SIZE must be between 0 and 1 inclusive.")
  )