(in-package diff-fn)

(defclass <tabular-difference-function> ()
  ((states :accessor states :initarg :states :type [numbered-set])
   (table :type (array float 2) :accessor table :initarg :table)
   (known :type (array boolean 2) :accessor known))
  (:documentation "Class <tabular-difference-function> 

Required initargs
:states - [numbered-set] of states

Optional
:table - the initial table (of type (array float 2))
"))

(defmethod initialize-instance :after ((f <tabular-difference-function>) &rest args &key (states nil supp))
  (assert supp nil ":states initarg must be provided for <tabular-difference-function>.")
  (let ((n (size states)))
    (if (slot-boundp f 'table)
	(setf (known f) (make-array (list n n) :element-type 'boolean :initial-element t))
      (setf (table f) (make-array (list n n) :element-type 'float :initial-element 42.42)
	    (known f) (make-array (list n n) :element-type 'boolean :initial-element nil)))))

(defmethod difference ((f <tabular-difference-function>) s s2 &optional (default-val nil) &aux (states (states f)))
  (let ((i (item-number s states))
	(j (item-number s2 states)))
    (if (aref (known f) i j)
	(aref (table f) i j)
      (or default-val (error 'unknown-difference :diff-fn f :state1 s :state2 s2)))))

(defmethod update ((f <tabular-difference-function>) s s2 target step-size &aux (states (states f)))
  (let ((i (item-number s states))
	(j (item-number s2 states)))
    (let ((old-val (if (aref (known f) i j) (aref (table f) i j) 0.0)))
      (setf (aref (table f) i j) (avg old-val target step-size))
      (setf (aref (known f) i j) t))))


(defmethod pprint-differences (str (d <tabular-difference-function>)
			       &aux (states (states d)))
  (pprint-logical-block (str nil :prefix "[" :suffix "]")
    (format str "Tabular difference function~2I")
    (set:do-elements (x states nil i)
      (set:do-elements (y states nil j)
	(handler-case
	    (let ((diff (difference d x y)))
	      (pprint-pop)
	      (format str "~:@_~W~10@T~W~10@T~6,2f" x y diff))
	  (unknown-difference ()))))))
	    

(set-pprint-dispatch '<tabular-difference-function> #'pprint-differences)


    
    
    
  