;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; rl-functions/policy/greedy-policy.lisp
;; Defines <greedy-policy> class
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package policy)

(defclass <greedy-policy> (<policy>)
  ((q-function :type q-fn:<q-function>
	       :reader q-function
	       :initarg :q-function)
   (unknown-state-behavior :accessor unknown-state-behavior
			   :initarg :unknown-state-behavior
			   :initform ':error)
   )
  (:documentation "<greedy-policy>.  Subclass of <policy>.  

Initargs
:q-function - make decisions by maximizing this q-function.
:unknown-state-behavior - When an unknown state (where the set of choices is itself not known to the q-function, or when all the choice q-values are unknown) is reached, an unknown-state error is signalled.  The make-choice method provides a restart called handle-unknown-state that decides what to do given the set of choices (which is provided by the caller when invoking the restart).  What happens in this restart depends on the :unknown-state-behavior initarg.  1) If it is ':error (the default), then the error is allowed to continue up the stack 2) If it is ':random, then a random choice is returned. 3) If it is ':first, then the first choice is returned."))


(defmethod make-choice ((d <greedy-policy>) omega &optional (choices nil choices-supp))
  (handler-case
      (q-fn:best-choice (q-function d) omega)
    ((or unknown-state unknown-state-action) (c)
      (if choices-supp
	  (let ((choice
		 (ecase (unknown-state-behavior d)
		   (:error (error c))
		   (:random (prob:sample-uniformly choices))
		   (:first (set:item 0 choices)))))
	    ;; (warn "Unknown state ~a.  Behavior at unknown states is ~a.  Chose ~a." omega (unknown-state-behavior d) choice)
	    choice)
	(error c)))))
	


(defmethod clone ((pol <greedy-policy>))
  (check-exact-class pol '<greedy-policy>)
  (make-instance '<greedy-policy> :q-function (clone (q-function pol)) :unknown-state-behavior (unknown-state-behavior pol)))

(defmethod print-object ((pol <greedy-policy>) str)
  (if *print-readably*
      (progn
	(check-exact-class pol '<greedy-policy>)
	(format str "#.(make-instance 'policy:<greedy-policy> :q-function ")
	(write (q-function pol) :stream str)
	(format str " :unknown-state-behavior ~W)" (unknown-state-behavior pol)))
               
    (call-next-method)))

		