(in-package q-learning)


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; individual Q-components
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defclass <decomposed-q-learning-component> (<q-learning>)
  ((global-q :accessor global-q :initarg :global-q)))


(defmethod inform-start-episode :after ((alg <decomposed-q-learning-component>) s2)
  (declare (ignore s2))
  (debug-msg alg "Inform-start-episode")
  (assert (member (q-fn alg) (q-fn::q-fns (global-q alg)))))

(defmethod inform-start-execution ((alg <decomposed-q-learning-component>))
  (debug-msg alg "Inform-start-execution")
  (assert (member (q-fn alg) (q-fn::q-fns (global-q alg)))))

(defmethod reset ((alg <decomposed-q-learning-component>))
  )

(defmethod reset :after ((alg <decomposed-q-learning-component>))
  (debug-msg alg "Reset")
  (assert (member (q-fn alg) (q-fn::q-fns (global-q alg)))))

(defmethod inform-env-step ((alg <decomposed-q-learning-component>) a r s2 term)
  (let ((prev-s (prev-s alg))
	(q (q-fn alg)))
    (let ((eta (lrate:get-rate (lrate alg) (cons prev-s a))))
       (assert (member q (q-fn::q-fns (global-q alg))))
      
      (condlet
       ((term (target r) (new-prev-s nil))
	(t (target (+ r (handler-case
			    (q-fn:evaluate 
			     q s2 (q-fn:best-choice (global-q alg) s2))
			  (q-fn:unknown-state-action () 0))))
	   (new-prev-s s2)))
       (let ((old-q (handler-case (q-fn:evaluate q prev-s a) (q-fn:unknown-state-action () 'unknown))))
	 (q-fn:update q prev-s a target eta)
	 (debug-msg alg "Inform-env-step: (~a, ~a, ~a, ~a) -> ~a.  Q-val: ~a -> ~a" prev-s a r s2 target old-q (q-fn:evaluate q prev-s a)))
       (setf (prev-s alg) new-prev-s)))))



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Overall decomposed learner
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defclass <decomposed-q-learning> (<q-learning>)
  ((reward-components :accessor comps :writer set-reward-components :initarg :reward-components)
   (init-q-fns :accessor init-q-fns :initarg :init-q-fns)
   (component-learners :accessor comp-learners))
  (:documentation "Class <decomposed-q-learning> (<q-learning>)

Initargs
:init-q-fns - list of q-function objects.  These are then used to initialize the q-components, or to reinitialize them when resetting the algorithm.
:reward-components - list of functions from s,a,r,s' to component rewards.  Guaranteed to be called in sequence every time.

NOTE: not tested yet!"))

(defmethod initialize-instance :after ((alg <decomposed-q-learning>) &rest args)
    (declare (ignore args))
    (reset alg))

(defmethod reset ((alg <decomposed-q-learning>) &aux (q-fns (clone (init-q-fns alg))))
  (let ((global-q (make-instance '<sum-q-function> :q-fns q-fns :strict nil)))
    (setf (comp-learners alg) (mapcar #'(lambda (q) (make-instance '<decomposed-q-learning-component> 
						      :global-q global-q :q-fn q))
				      q-fns))
    (set-q-fn global-q alg)))


(defmethod inform-env-step ((alg <decomposed-q-learning>) a r s2 term &aux (s (prev-s alg)))
  (let ((rewards (mapcar #'(lambda (comp) (funcall comp s a r s2)) (comps alg))))
    (map nil #'(lambda (l r) (inform-env-step l a r s2 term))
	 (comp-learners alg) rewards)))


