(defpackage env-observer
  (:documentation "Package env-observer.  Defines <env-observer>, a type of observer that prints out what happens in the environment to a stream.

Exports
-------
<env-observer>
make-env-observer")
  (:use cl
	rl-obs
	utils)
  (:export 
   <env-observer>
   make-env-observer))

(in-package env-observer)
  
(defclass <env-observer> (<rl-observer>)
  ((output-stream :type (or symbol stream)
		  :reader str
		  :initarg :str
		  :writer set-output-stream
		  :initform t)
   (prompt-after-actions :type boolean
			 :accessor prompt?
			 :initarg :prompt?
			 :initform nil)
   (prompt-this-episode :type boolean
			:accessor prompt-this-ep
			:initform nil)
   (prompt-after-episodes :type boolean :accessor prompt-after-episodes :initarg :prompt-after-episodes
			  :initform nil)
   (eps :type fixnum
	:accessor eps
	:initform 0)
   (steps :type fixnum
	  :accessor steps))
  (:documentation "Subclass of <rl-observer> whose sole function is to show what happens in the environment.  Create instances using make-env-observer"))




(defun make-env-observer (&optional (stream t) (prompt? nil))
  "make-env-observer &optional (OUTPUT-STREAM t) (PROMPT? nil).  Make an <env-observer>.  When PROMPT? is true, prompts user to press enter after each step."
  (make-instance '<env-observer> :str stream :prompt? prompt? ))

(defmethod inform-start-episode ((obs <env-observer>) s)
  (let ((str (str obs)))
    (when str
      (fresh-line str)
      (pprint-logical-block (str nil)
	(format str "~:@_Environment reset to:~5I~:@_~W" s))))
  (vector-push-extend (list s) rl-user:*hist*)
  (setf (prompt-this-ep obs) t
	(steps obs) 0))
  

(defmethod inform-env-step ((obs <env-observer>) act rew to term)
  (setf (cdr (slast rl-user:*hist*)) act)
  (vector-push-extend (list to) rl-user:*hist*)
  (let ((str (str obs)))
    (when str
      (fresh-line str)
      (pprint-logical-block (str nil)
	(format str "~:@_Episode ~W Step ~W~5I~:@_Action:~20T~W~:@_Reward:~20T~W~:@_New state:~20T~W" 
	    (eps obs) (steps obs) act rew to)))
    (when (and (prompt? obs) (prompt-this-ep obs))
      (let ((c (prompt "~&Press enter to continue or \"skip\" to skip to end of this episode. ")))
	(when (equal c "skip") (setf (prompt-this-ep obs) nil)))))
  (incf (steps obs))
  (when term
    (awhen (str obs)
      (format it "~&Episode complete.")
      (awhen (prompt-after-episodes obs)
	(unless (y-or-n-p "  Start new episode? ")
	  (abort)
	  )))
    (incf (eps obs)))
  (values))

  
  