(defpackage message-logger
  (:documentation "
Types
-----
<message-logger>")
  (:use cl
	utils
	rl-obs)
  (:export <message-logger>))
  

(in-package message-logger)

(defclass <message-logger> (<rl-observer>)
  ((str :type stream :initarg :str :reader str :initform t)
   (logger-name :type string :initarg :name :reader name :initform "message-logger"))
  
  (:documentation "A kind of <rl-observer> that just prints start-execution, finish-execution, start-episode, and env-step messages to a stream.

Initargs
:str - the stream to print to.  Defaults to t.
:name - the name, which serves to identify the logger if there are multiple ones.  Defaults to the the name of the symbol 'message-logger.
"))


(defmethod inform-start-execution ((obs <message-logger>))
  (let ((str (str obs)))
    (fresh-line str)
    (pprint-logical-block (str nil)
      (format str "~:@_~A received start-execution message" (name obs)))))

(defmethod inform-finish-execution ((obs <message-logger>))
  (let ((str (str obs)))
    (fresh-line str)
    (pprint-logical-block (str nil)
      (format str "~:@_~A received finish-execution message" (name obs)))))

(defmethod inform-start-episode ((obs <message-logger>) s)
  (let ((str (str obs)))
    (fresh-line str)
    (pprint-logical-block (str nil)
      (format str "~:@_~A received start-episode message for~5I~:@_State: ~W" (string-capitalize (name obs)) s))))

(defmethod inform-env-step ((obs <message-logger>) a r s2 term)
  (let ((str (str obs)))
    (fresh-line str)
    (pprint-logical-block (str nil)
      (format str "~:@_~A received env-step message for~5I~:@_action: ~W~:@_reward: ~5,2F~:@_term: ~W~:@_s2: ~W"
	      (name obs) a r term s2))))