(defpackage reinforcement-learning
  (:documentation "Internal package for (flat) reinforcement learning.
")
  (:nicknames rl)
  (:export
   learn
   on-policy-learning
   evaluate
   io-interface
   <rl-control>
   reinforcement-learning
   gather-trajectories
	     batch-learning
	     random
	     set-hist-collect
	     get-policy-hist
	     get-value-fn-hist
	     get-q-hist
	     get-advantage-hist
	     current-q-fn
	     current-advantage-fn
	     current-value-fn
	     current-policy
	     get-mdp-hist
	     current-mdp
	     *hist*
	     reset
	     debug-str
	     set-debug-str
	     learn
	     *on-policy-learning*
	     evaluate
	     io-interface

	     reinforcement-learning
	     gather-trajectories
	     batch-learning
	     random
	     set-hist-collect
	     get-policy-hist
	     get-value-fn-hist
	     get-q-hist
   <rl-observer>
   <learning-algorithm>
	     <on-policy-learning-algorithm>
	     <q-learning-algorithm>
	     <policy-learning-algorithm>
	     <value-learning-algorithm>
	     <advantage-learning-algorithm>
	     defmessage
	     debug-msg
	     inform-start-execution
	     inform-finish-execution
	     inform-start-episode
	     inform-env-step
	     reset
	     knowledge-state
	     get-q-fn
	     get-policy
	     get-mdp
	     get-value-fn
	     get-advantage-fn
	     current-env-step
	     current-episode-step
	     on-policy-learning-in-progress
	     debug-str
	     set-debug-str



)


  (:use common-lisp
	utils
	set
	prob)
)

(defpackage rl-user
  (:documentation "Package rl-user.  Package used when running rl algorithms in environments.

Exported symbols
==================

Running
-------
learn
on-policy-learning
evaluate  
io-interface
<rl-control>
reinforcement-learning
gather-trajectories
batch-learning
random

General operations on learning algorithms
-----------------------------------------
hist
get-policy-hist
get-q-hist
get-advantage-hist
current-q-fn
current-policy
current-advantage-fn
current-value-fn
get-value-fn-hist
get-mdp-hist
get-advantage-hist
current-mdp
set-hist-collect
reset
debug-str
set-debug-str


Other
-----
*hist*


Related packages
================

Other observers 
---------------
- env-observer
- progress-printer
- q-learning
- gold-standard
- stat-gatherer
")
  (:use common-lisp
	rl)
  (:export   learn
	     on-policy-learning
	     evaluate
	     io-interface
	     <rl-control>
	     reinforcement-learning
	     gather-trajectories
	     batch-learning
	     random
	     set-hist-collect
	     get-policy-hist
	     get-value-fn-hist
	     get-q-hist
	     get-advantage-hist
	     current-q-fn
	     current-advantage-fn
	     current-value-fn
	     current-policy
	     get-mdp-hist
	     current-mdp
	     *hist*
	     reset
	     debug-str
	     set-debug-str
	     ))

	     


(defpackage rl-observer
  (:nicknames rl-obs)
  (:documentation "Package rl-observer.  Used when making new observers/learning algorithms/stats gatherers.

Types
-----
<rl-observer>
<learning-algorithm>
<on-policy-learning-algorithm>
<q-learning-algorithm>
<policy-learning-algorithm>
<value-learning-algorithm>
<advantage-learning-algorithm>

Macro
-----
defmessage
debug-msg

Messages
--------
inform-start-execution
inform-finish-execution
inform-start-episode
inform-env-step

Learning algorithms must implement some subset of (depending on specific type)
------------------------------------------------------------------------------
reset
knowledge-state
get-q-fn (q-learning algorithms)
get-mdp (model-learning algorithms)
get-policy (policy-learning algorithms)
get-value-fn (value-learning algorithms)
get-advantage-fn (advantage learning algorithms)
make-choice (on-policy algorithms)

Functions for use by learning algorithms
----------------------------------------
current-env-step
current-episode-step
on-policy-learning-in-progress

Other accessors for learning algorithms
---------------------------------------
debug-str
set-debug-str

")
  (:use common-lisp
	rl
	policy)
  (:export
   <rl-observer>
   <learning-algorithm>
	     <on-policy-learning-algorithm>
	     <q-learning-algorithm>
	     <policy-learning-algorithm>
	     <value-learning-algorithm>
	     <advantage-learning-algorithm>
	     defmessage
	     debug-msg
	     inform-start-execution
	     inform-finish-execution
	     inform-start-episode
	     inform-env-step
	     reset
	     knowledge-state
	     get-q-fn
	     get-policy
	     get-mdp
	     get-value-fn
	     get-advantage-fn
	     current-env-step
	     current-episode-step
	     on-policy-learning-in-progress
	     debug-str
	     set-debug-str
	     make-choice
	     ))




(defvar rl:*hist* nil
  "When the IO interface is running, holds the trajectory of joint states seen during execution.  This provides a quick way to generate env state objects - just use the IO interface to get where you want, then stop and use the last item of *hist*.  Set to a fresh vector on each run (so the old *hist* can just be copied to some other place and won't get overwritten when a new run happens).")
(defvar rl:*on-policy-learning* nil)


(in-package cl-user)
	     
