(defpackage test-rl
  (:use common-lisp
	utils
	rl-user
	q-learning
	td-taxi-env
	gold-standard))

(in-package test-rl)

(format t "~&Testing RL Code~&Note: this might occasionally hang in the early stages if policy iteration gets into an infinite loop.")
(fill-format-nl #\= 60)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; create environment, compute ground truth
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defvar env (td-taxi-env:make-example-env1))
;(setf v-true (make-array 3 :initial-contents '(22.3 19.8 0)))
;;(setf q-true (dp:q-from-v mdp-test-envs:*test-mdp-1* v-true))
;(setf e (make-instance 'mdp-env:<mdp-env> :mdp mdp-test-envs:*test-mdp-1*
;			       :init-dist #(.5 .5)))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;; 
;; learning exp
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


(defun evaluate-alg (alg)
  (evaluate env (get-policy-hist alg) :num-steps 25 
	    :num-trials 10 :env-display-stream nil 
	    :pause-after-actions nil))





(defvar *num-steps-learning* 20000)
(defvar *hist-length* 100)

(defvar featurizer)
(setf featurizer (td-taxi-flat-lfa:make-taxi-featurizer env))
(defvar lq
  (make-instance 'q-fn:<env-q-function> :env env :env-name 'env :featurizer featurizer
		 :featurizer-name 'featurizer
		 :fn-approx (make-instance 'fn-approx:<linear-fn-approx> :dim 7)))

(defvar q-learner (make-instance '<q-learning> :env env :lrate .01 :discount 1.0 :unknown-state-behavior ':random))
(defvar lfa-q-learner (make-instance '<q-learning> :lrate .01 :discount 1.0 :q-fn lq :hist-out-dir "test/temp"))
(defvar api-learner (make-instance 'api:<approx-pol-it> :env env :pol-imp-int 100 :pol-switch 500))
(defvar f nil)
;(with-outfile (f "scratch/test-gs.out")
(defvar gs-learner (make-instance '<gold-standard> :debug-str f :unknown-state-behavior ':random))
(defvar env-obs (env-observer:make-env-observer f))
(learn env 'random (list q-learner lfa-q-learner gs-learner) *num-steps-learning*
       :hist-length *hist-length* :ep-print-inc 10 :step-print-inc 100)
; (on-policy-learning e api-learner *num-steps-learning*
; 		    :hist-length *hist-length* :ep-print-inc 100 :step-print-inc 100)

(defvar qh (get-q-hist q-learner))
(defvar lqh (get-q-hist lfa-q-learner))
(defvar gh (get-q-hist gs-learner))
; (setf ah (get-q-hist api-learner))
      



(format t "~&Q-learning algorithm learning curve is~&~W"
	(defvar q-rews (evaluate-alg q-learner)))

(format t "~&Linearly approximated Q-learning learning curve is~&~W"
	(defvar lq-rews (evaluate-alg lfa-q-learner)))

(format t "~&Gold standard algorithm learning curve is~&~W"
 	  (defvar gs-rews (evaluate-alg gs-learner)))

; (format t "~&Approximate policy iteration learning curve is~&~W"
;	(setf api-rews (evaluate-alg api-learner)))

(defun proportion (f &rest args)
  (/ (let ((sum 0))
       (apply #'map nil #'(lambda (&rest a) (incf sum (indicator (apply f a)))) args)
       sum)
     (length (first args))))

(defparameter *mostly* .75)
(defun mostly-in-range (s bounds)
  (dbind (l h) bounds
    (> (proportion #'(lambda (x) (between x l h)) s) *mostly*)))

(defun mostly-greater-than (s1 s2)
  (> (proportion #'> s1 s2) *mostly*))

(tests 
 "Checking if learning curves are in expected range"
 (q-rews '(-5 0) #'mostly-in-range  )
 (gs-rews '(0 1 ) #'mostly-in-range  ))
       
     


(reset lfa-q-learner)

(in-package common-lisp-user)    










